# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from os import stat
import re

from sos_tests import StageOneReportTest, StageTwoReportTest


class ReportWithMask(StageOneReportTest):
    """Testing around basic --clean/--mask usage and expectations

    :avocado: tags=stageone
    """

    sos_cmd = '--mask -o host,networking'
    hosts_obfuscated = None

    def pre_sos_setup(self):
        # obfuscate a random word from /etc/hosts and ensure the updated
        # sanitised file has same permissions (a+r)
        try:
            with open('/etc/hosts', encoding='utf-8') as fp:
                self.obsfuncated = fp.read().strip('#\n').split()[-1]
        except (FileNotFoundError, IndexError) as e:
            self.warning(f"Unable to process /etc/hosts: {e}")
        if self.hosts_obfuscated:
            self.sos_cmd += f' --keywords={self.hosts_obfuscated}'

    def test_mask_was_run(self):
        self.assertOutputContains('Beginning obfuscation')
        self.assertOutputContains('Obfuscation completed')

    def test_private_map_was_generated(self):
        self.assertOutputContains(
            'A mapping of obfuscated elements is available at')
        map_file = re.findall(
            '/.*sosreport-.*-private_map', self.cmd_output.stdout)[-1]
        self.assertFileExists(map_file)

    def test_tarball_named_obfuscated(self):
        self.assertTrue('obfuscated' in self.archive)

    def test_archive_type_correct(self):
        self.assertSosLogContains('Loaded .* as type sos report directory')

    def test_localhost_was_obfuscated(self):
        self.assertFileHasContent('hostname', 'host0')

    def test_ip_address_was_obfuscated(self):
        # Note: do not test for starting with the 100.* block here, as test
        # machines may have /32 addresses. Instead, test that the actual
        # IP address is not present
        self.assertFileNotHasContent(
            'ip_addr',
            self.sysinfo['pre']['networking']['ip_addr']
        )

    def test_loopback_was_not_obfuscated(self):
        self.assertFileHasContent('ip_addr', '127.0.0.1/8')

    def test_mac_addrs_were_obfuscated(self):
        content = self.get_file_content(
            'sos_commands/networking/ip_maddr_show'
        )
        for line in content.splitlines():
            if line.strip().startswith('link'):
                mac = line.strip().split()[1]
                assert \
                    mac.startswith('53:4f:53'), \
                    f"Found unobfuscated mac addr {mac}"

    def test_perms_unchanged_on_modified_file(self):
        if self.hosts_obfuscated:
            imode_orig = stat('/etc/hosts').st_mode
            imode_obfuscated = stat(
                self.get_name_in_archive('etc/hosts')).st_mode
            self.assertEqual(imode_orig, imode_obfuscated)


class ReportWithUserCustomisations(StageOneReportTest):
    """Testing for 1) obfuscated keywords provided by the user (--keywords
    option), and 2) skipping to clean specific files (--skip-cleaning-files
    option)

    :avocado: tags=stageone
    """

    sos_cmd = ('--clean -o filesys,kernel --keywords=fstab,Linux,tmp,'
               'BOOT_IMAGE,fs.dentry-state --skip-cleaning-files '
               'proc/cmdline,sos_commands/*/sysctl* --no-update')

    # Will the 'tmp' be properly treated in path to working dir without
    # raising an error?
    # To make this test effective, we assume the test runs on a system / with
    # Policy returning '/var/tmp' as temp.dir
    def test_keyword_in_tempdir_path(self):
        self.assertOutputContains(
            'Your sos report has been generated and saved in:'
        )
        self.assertTrue('tmp/' in self.archive)

    # Ok, sort of cheesy here but this does actually test filename changes on
    # a file common to all distros
    def test_filename_obfuscated(self):
        self.assertFileNotCollected('/etc/fstab')
        self.assertFileGlobInArchive('/etc/obfuscatedword*')

    def test_keyword_obfuscated_in_file(self):
        self.assertFileNotHasContent('sos_commands/kernel/uname_-a', 'Linux')

    def test_skip_cleaning_single_file(self):
        self.assertFileHasContent('proc/cmdline', 'BOOT_IMAGE')

    def test_skip_cleaning_glob_file(self):
        self.assertFileHasContent(
            'sos_commands/kernel/sysctl_-a',
            'fs.dentry-state'
        )


class DefaultRemoveBinaryFilesTest(StageTwoReportTest):
    """Testing that binary files are removed by default

    :avocado: tags=stagetwo
    """

    files = [('binary_test.tar.xz', '/var/log/binary_test.tar.xz')]
    install_plugins = ['binary_test']
    sos_cmd = '--clean -o binary_test,kernel,host'

    def test_binary_removed(self):
        self.assertFileNotCollected('var/log/binary_test.tar.xz')

    def test_binaries_removed_reported(self):
        self.assertOutputContains(r'\[removed .* unprocessable files\]')


class KeepBinaryFilesTest(StageTwoReportTest):
    """Testing that --keep-binary-files will function as expected

    :avocado: tags=stagetwo
    """

    files = [('binary_test.tar.xz', '/var/log/binary_test.tar.xz')]
    install_plugins = ['binary_test']
    sos_cmd = '--clean --keep-binary-files -o binary_test,kernel,host'

    def test_warning_message_shown(self):
        self.assertOutputContains(
            'WARNING: binary files that potentially contain sensitive '
            'information will NOT be removed from the final archive'
        )

    def test_binary_is_in_archive(self):
        self.assertFileCollected('var/log/binary_test.tar.xz')

    def test_no_binaries_reported_removed(self):
        self.assertOutputNotContains(r'\[removed .* unprocessable files\]')



# Copyright 2020 Red Hat, Inc. Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re

from sos.cleaner.parsers import SoSCleanerParser
from sos.cleaner.mappings.mac_map import SoSMacMap


# aa:bb:cc:fe:ff:dd:ee:ff
IPV6_REG_8HEX = (
    r'((?<!([0-9a-fA-F\'\"]:)|::)([^:|-])?([0-9a-fA-F]{2}(:|-)){7}'
    r'[0-9a-fA-F]{2}(\'|\")?(\/|\,|\-|\.|\s|$))'
)
# aabb:ccee:ddee:ffaa
# - but disallow "substrings"
#   - but allow fe80: or fe80:: prefix for link-local
IPV6_REG_4HEX = (
        r'((?<!(?:([.|^|\b]{5}\w|[.|^|\b]fe80:|fe80::)))'
        r'(([0-9a-fA-F]{4}:){3}[0-9a-fA-F]{4})(?!\w))|'
        r'((?<!\w)(([0-9a-fA-F]{4}-){3}[0-9a-fA-F]{4})(?!\w))'
)
# aa:bb:cc:dd:ee:ff avoiding ipv6 substring matches
IPV4_REG = (
    r'((?<!([0-9a-fA-F\'\"]:)|::)'
    r'(([^:\-])?(([0-9a-fA-F]{2}([:\-\_])){5,6}([0-9a-fA-F]{2}))))'
)


class SoSMacParser(SoSCleanerParser):
    """Handles parsing for MAC addresses"""

    name = 'MAC Parser'
    regex_patterns = [
        IPV6_REG_8HEX,
        IPV6_REG_4HEX,
        IPV4_REG
    ]
    obfuscated_patterns = (
        '53:4f:53',
        '534f:53'
    )
    parser_skip_files = [
        'sos_commands/.*/modinfo.*'
    ]
    map_file_key = 'mac_map'
    compile_regexes = False

    def __init__(self, config, workdir, skip_cleaning_files=[]):
        self.mapping = SoSMacMap(workdir)
        super().__init__(config, skip_cleaning_files)

    def reduce_mac_match(self, match):
        """Strips away leading and trailing non-alphanum characters from any
        matched string to leave us with just the bare MAC addr
        """
        while not match[0] in '0123456789abcdefABCDEF':
            match = match[1:]
        while not match[-1] in '0123456789abcdefABCDEF':
            match = match[0:-1]
        # just to be safe, call strip() to remove any padding
        return match.strip()

    def _parse_line(self, line):
        count = 0
        for pattern in self.regex_patterns:
            matches = [m[0] for m in re.findall(pattern, line, re.I)]
            if matches:
                count += len(matches)
                for match in matches:
                    stripped_match = self.reduce_mac_match(match)
                    if stripped_match.startswith(self.obfuscated_patterns):
                        # avoid double scrubbing
                        continue
                    new_match = self.mapping.get(stripped_match)
                    line = line.replace(stripped_match, new_match)
        return line, count



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos_tests import StageOneReportTest


class rhbz1965001(StageOneReportTest):
    """
    Copying /proc/sys/vm/{compact_memory,drop_caches} must ignore SELinux
    context, otherwise an attempt to set the context to files under some
    directories like /tmp raises an AVC denial, and an ERROR
    "Unable to add '...' to archive: [Errno 13] Permission denied: '...'
    is raise.

    https://bugzilla.redhat.com/show_bug.cgi?id=1965001

    :avocado: enable
    :avocado: tags=stageone
    """

    sos_cmd = '-o system'
    # it is crucial to run the test case with --tmp-dir=/tmp/... as that is
    # (an example of) directory exhibiting the relabel permission deny.
    # /var/tmp directory allows those relabels.
    #
    # the directory shouldn't exist at this moment, otherwise
    # "check to prevent multiple setUp() runs" in sos_tests.py would fail
    _tmpdir = '/tmp/rhbz1965001_avocado_test'

    def test_no_permission_denied(self):
        self.assertSosLogNotContains("Permission denied")



# Copyright (C) 2025 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Snapm(Plugin, RedHatPlugin):

    short_desc = 'Snapsot manager'

    plugin_name = 'snapm'
    profiles = ('boot', 'system', 'storage')

    packages = (
        'python3-snapm',
        'snapm',
    )

    def setup(self):
        self.add_copy_spec([
            "/etc/snapm",
        ])

        self.add_cmd_output([
            "snapm snapset list",
            "snapm snapshot list",
            "snapm plugin list",
            "snapm schedule list",
            "snapm -vv --debug=all snapset show",
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import unittest

from avocado.utils import distro

from sos.policies import Policy, import_policy
from sos.policies.distros import LinuxPolicy
from sos.policies.package_managers import PackageManager, MultiPackageManager
from sos.policies.package_managers.rpm import RpmPackageManager
from sos.policies.package_managers.dpkg import DpkgPackageManager
from sos.report.plugins import (Plugin, IndependentPlugin,
                                RedHatPlugin, DebianPlugin)


class FauxPolicy(Policy):
    distro = "Faux"

    @classmethod
    def check(cls, remote=''):
        return False


class FauxLinuxPolicy(LinuxPolicy):
    distro = "FauxLinux"

    @classmethod
    def check(cls, remote=''):
        return False

    @classmethod
    def set_forbidden_paths(cls):
        return ['/etc/secret']


class FauxPlugin(Plugin, IndependentPlugin):
    pass


class FauxRedHatPlugin(Plugin, RedHatPlugin):
    pass


class FauxDebianPlugin(Plugin, DebianPlugin):
    pass


class PolicyTests(unittest.TestCase):

    def test_independent_only(self):
        p = FauxPolicy()
        p.valid_subclasses = []

        self.assertTrue(p.validate_plugin(FauxPlugin))

    def test_forbidden_paths_building(self):
        p = FauxLinuxPolicy(probe_runtime=False)
        self.assertTrue('*.pyc' in p.forbidden_paths)
        self.assertTrue('/etc/passwd' in p.forbidden_paths)
        self.assertTrue('/etc/secret' in p.forbidden_paths)

    def test_redhat(self):
        p = FauxPolicy()
        p.valid_subclasses = [RedHatPlugin]

        self.assertTrue(p.validate_plugin(FauxRedHatPlugin))

    def test_debian(self):
        p = FauxPolicy()
        p.valid_subclasses = [DebianPlugin]

        self.assertTrue(p.validate_plugin(FauxDebianPlugin))

    def test_fails(self):
        p = FauxPolicy()
        p.valid_subclasses = []

        self.assertFalse(p.validate_plugin(FauxDebianPlugin))

    def test_can_import(self):
        self.assertTrue(import_policy('redhat') is not None)

    def test_cant_import(self):
        self.assertTrue(import_policy('notreal') is None)


class PackageManagerTests(unittest.TestCase):

    def setUp(self):
        self.pm = PackageManager()

    def test_default_all_pkgs(self):
        self.assertEqual(self.pm.packages, {})

    def test_default_all_pkgs_by_name(self):
        self.assertEqual(self.pm.all_pkgs_by_name('doesntmatter'), [])

    def test_default_all_pkgs_by_name_regex(self):
        self.assertEqual(
            self.pm.all_pkgs_by_name_regex('.*doesntmatter$'), [])

    def test_default_pkg_by_name(self):
        self.assertEqual(self.pm.pkg_by_name('foo'), None)


class RpmPackageManagerTests(unittest.TestCase):

    def setUp(self):
        if distro.detect().name not in ['fedora', 'centos', 'rhel']:
            self.skipTest('Not running on an RPM distribution')
        self.pm = RpmPackageManager()

    def test_load_all_packages(self):
        self.assertNotEqual(self.pm.packages, {})

    def test_pkg_is_formatted(self):
        kpkg = self.pm.pkg_by_name('coreutils')
        self.assertIsInstance(kpkg, dict)
        self.assertIsInstance(kpkg['version'], list)
        self.assertEqual(kpkg['pkg_manager'], 'rpm')


class DpkgPackageManagerTests(unittest.TestCase):

    def setUp(self):
        if distro.detect().name not in ['Ubuntu', 'debian']:
            self.skipTest('Not running on a dpkg distribution')
        self.pm = DpkgPackageManager()

    def test_load_all_packages(self):
        self.assertNotEqual(self.pm.packages, {})

    def test_pkg_is_formatted(self):
        kpkg = self.pm.pkg_by_name('coreutils')
        self.assertIsInstance(kpkg, dict)
        self.assertIsInstance(kpkg['version'], list)
        self.assertEqual(kpkg['pkg_manager'], 'dpkg')


class MultiPackageManagerTests(unittest.TestCase):

    def setUp(self):
        self.pm = MultiPackageManager(primary=RpmPackageManager,
                                      fallbacks=[DpkgPackageManager])

    def test_load_all_packages(self):
        self.assertNotEqual(self.pm.packages, {})

    def test_pkg_is_formatted(self):
        kpkg = self.pm.pkg_by_name('coreutils')
        self.assertIsInstance(kpkg, dict)
        self.assertIsInstance(kpkg['version'], list)
        _local = distro.detect().name
        if _local in ['Ubuntu', 'debian']:
            self.assertEqual(kpkg['pkg_manager'], 'dpkg')
        else:
            self.assertEqual(kpkg['pkg_manager'], 'rpm')


if __name__ == "__main__":
    unittest.main()

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class VectorDev(Plugin, IndependentPlugin):

    short_desc = 'A tool for building observability pipelines'

    plugin_name = "vectordev"
    profiles = ('observability',)
    files = ('/etc/vector/',)

    def setup(self):
        self.add_copy_spec([
            "/etc/vector/"
        ])

    def postproc(self):

        vector_config_path = "/etc/vector/*"
        protect_keys = [
            "auth.password",
            "auth.token",
            "tls.key_pass",
        ]

        # Redact yaml and ini style "key (:|=) value".
        keys_regex = fr"(^\s*({'|'.join(protect_keys)})\s*(:|=)\s*)(.*)"
        sub_regex = r"\1*********"
        self.do_path_regex_sub(vector_config_path, keys_regex, sub_regex)
        # Redact certificates
        self.do_file_private_sub(vector_config_path)


# vim: et ts=4 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, UbuntuPlugin, SoSPredicate
from sos.utilities import is_executable


class Ubuntu(Plugin, UbuntuPlugin):

    short_desc = 'Ubuntu specific information'

    plugin_name = 'ubuntu'
    profiles = ('system',)

    packages = (
        'ubuntu-advantage-tools',
        'ubuntu-pro-client',
        'update-manager-core',
    )

    def setup(self):
        self.add_cmd_output([
            "ubuntu-security-status --thirdparty --unavailable",
            "hwe-support-status --verbose",
        ])

        if is_executable('ua'):
            ua_tools_status = 'ua status'
        elif is_executable('pro'):
            ua_tools_status = 'pro status'
        else:
            ua_tools_status = 'ubuntu-advantage status'
        ua_pred = SoSPredicate(self, kmods=['tls'])
        self.add_cmd_output(ua_tools_status,
                            pred=ua_pred, changes=True)
        self.add_cmd_output(f"{ua_tools_status} --format json",
                            pred=ua_pred, changes=True)

        self.add_copy_spec("/etc/ubuntu-advantage/uaclient.conf")

        if not self.get_option("all_logs"):
            self.add_copy_spec([
               "/var/log/ubuntu-advantage.log",
               "/var/log/ubuntu-advantage.log.1",
               "/var/log/ubuntu-advantage.log.2*",
               "/var/log/ubuntu-advantage-apt-hook.log",
               "/var/log/ubuntu-advantage-apt-hook.log.1",
               "/var/log/ubuntu-advantage-apt-hook.log.2*",
               "/var/log/ubuntu-advantage-timer.log",
               "/var/log/ubuntu-advantage-timer.log.1",
               "/var/log/ubuntu-advantage-timer.log.2*",
            ])
        else:
            self.add_copy_spec([
               "/var/log/ubuntu-advantage.log*",
               "/var/log/ubuntu-advantage-apt-hook.log*",
               "/var/log/ubuntu-advantage-timer.log*",
            ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2024 Red Hat, Inc., Jose Castillo <jcastillo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import (Plugin, IndependentPlugin)


class Kea(Plugin, IndependentPlugin):
    """
    Kea is the next generation of DHCP software, developed by Internet
    Systems Consortium (ISC). It supports both the DHCPv4 and DHCPv6 protocols
    along with their extensions, e.g. prefix delegation and dynamic updates to
    DNS.
    """
    short_desc = 'Kea DHCP and DDNS server from ISC'

    plugin_name = "kea"
    packages = ("kea", "kea-common",)
    services = ('kea-ctrl-agent', 'kea-dhcp-ddns-server',
                'kea-dhcp4-server', 'kea-dhcp6-server',)

    def setup(self):
        self.add_copy_spec([
            "/etc/kea/*",
        ])
        self.add_cmd_output([
            "keactrl status",
        ])

    def postproc(self):
        """ format is "password": "kea", """
        self.do_path_regex_sub(
            '/etc/kea/*',
            r'(^\s*"password":\s*)(".*"),',
            r'\1********'
        )

# vim: set et ts=4 sw=4 :



# Copyright 2024 Red Hat, Inc. Jose Castillo <jcastillo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
import re
import logging

from getpass import getpass
from sos import _sos as _
from sos.utilities import is_executable, TIMEOUT_DEFAULT

try:
    import requests
    REQUESTS_LOADED = True
except ImportError:
    REQUESTS_LOADED = False

try:
    import boto3
    BOTO3_LOADED = True
except ImportError:
    BOTO3_LOADED = False


class UploadTarget():
    """
    This class is designed to upload files to a distribution
    defined location. These files can be either sos reports,
    sos collections, or other kind of files like: vmcores,
    application cores, logs, etc.

    """

    desc = """
            Upload a file (can be an sos report, a must-gather, or others) to
             a distribution defined remote location
            """
    # _ prefixed class attrs are used for storing any vendor-defined defaults
    # the non-prefixed attrs are used by the upload methods, and will be set
    # to the cmdline/config file values, if provided. If not provided, then
    # those attrs will be set to the _ prefixed values as a fallback.
    # TL;DR Use _upload_* for target default values, use upload_* when wanting
    # to actual use the value in a method/override
    upload_target_name = "Generic Upload"
    upload_target_id = "generic"
    _upload_file = None
    _upload_url = None
    _upload_directory = '/'
    _upload_user = None
    _upload_password = None
    _upload_method = None
    _upload_s3_endpoint = 'https://s3.amazonaws.com'
    _upload_s3_bucket = None
    _upload_s3_access_key = None
    _upload_s3_secret_key = None
    _upload_s3_region = None
    _upload_s3_object_prefix = ''
    upload_url = None
    upload_user = None
    upload_password = None
    upload_s3_endpoint = None
    upload_s3_bucket = None
    upload_s3_access_key = None
    upload_s3_secret_key = None
    upload_s3_region = None
    upload_s3_object_prefix = None
    upload_target = None

    arg_defaults = {
        'upload_file': '',
        'case_id': '',
        'low_priority': False,
        'profiles': [],
        'upload_url': None,
        'upload_directory': None,
        'upload_user': None,
        'upload_pass': None,
        'upload_method': 'auto',
        'upload_no_ssl_verify': False,
        'upload_protocol': 'auto',
        'upload_s3_endpoint': None,
        'upload_s3_region': None,
        'upload_s3_bucket': None,
        'upload_s3_access_key': None,
        'upload_s3_secret_key': None,
        'upload_s3_object_prefix': None,
        'upload_target': None,
    }

    def __init__(self, parser=None, args=None, cmdline=None):

        self.ui_log = logging.getLogger('sos_ui')
        self.parser = parser
        self.cmdline = cmdline
        self.args = args

    def check_distribution(self):
        """This should be overridden by upload targets

        This is called by sos upload on each target type that exists, and
        is meant to return True when the upload target matches a criteria
        that indicates that is the local upload target that should be used.

        Only the first upload target to determine a match is selected"""
        return False

    def get_target_id(self):
        return self.upload_target_id

    @classmethod
    def name(cls):
        """Returns the upload target's name as a string."""
        if cls.upload_target_name:
            return cls.upload_target_name
        return cls.__name__.lower()

    def get_commons(self):
        return {
            'cmdlineopts': self.hook_commons['cmdlineopts'],
            'policy': self.hook_commons['policy'],
            'case_id': self.hook_commons['cmdlineopts'].case_id,
            'upload_directory': self.hook_commons['cmdlineopts']
            .upload_directory
        }

    def set_commons(self, commons):
        """Set common host data for the Upload targets
            to reference
        """
        self.commons = commons

    def pre_work(self, hook_commons):

        self.hook_commons = hook_commons
        self.commons = self.get_commons()
        cmdline_opts = self.commons['cmdlineopts']
        policy = self.commons['policy']

        if cmdline_opts.low_priority:
            policy._configure_low_priority()

        # Set the cmdline settings to the class attrs that are referenced later
        # The target default '_' prefixed versions of these are untouched to
        # allow fallback
        self.upload_url = cmdline_opts.upload_url
        self.upload_user = cmdline_opts.upload_user
        self.upload_directory = cmdline_opts.upload_directory
        self.upload_password = cmdline_opts.upload_pass
        self.upload_archive_name = ''

        self.upload_s3_endpoint = cmdline_opts.upload_s3_endpoint
        self.upload_s3_region = cmdline_opts.upload_s3_region
        self.upload_s3_access_key = cmdline_opts.upload_s3_access_key
        self.upload_s3_bucket = cmdline_opts.upload_s3_bucket
        self.upload_s3_object_prefix = cmdline_opts.upload_s3_object_prefix
        self.upload_s3_secret_key = cmdline_opts.upload_s3_secret_key

        # set or query for upload credentials; this needs to be done after
        # setting case id, as below methods might rely on detection of it
        if not cmdline_opts.batch and not \
                cmdline_opts.quiet:
            # Targets will need to handle the prompts for user information
            if self.get_upload_url() and \
                    not cmdline_opts.upload_protocol == 's3':
                self.prompt_for_upload_user()
                self.prompt_for_upload_password()
            elif cmdline_opts.upload_protocol == 's3':
                self.prompt_for_upload_s3_bucket()
                self.prompt_for_upload_s3_endpoint()
                self.prompt_for_upload_s3_access_key()
                self.prompt_for_upload_s3_secret_key()
            self.ui_log.info('')

    def prompt_for_upload_s3_access_key(self):
        """Should be overridden by targets to determine if an access key needs
        to be provided for upload or not
        """
        if not self.get_upload_s3_access_key():

            msg = (
                "Please provide the upload access key for bucket"
                f" {self.get_upload_s3_bucket()} via endpoint"
                f" {self.get_upload_s3_endpoint()}: "
            )
            self.upload_s3_access_key = input(_(msg))

    def prompt_for_upload_s3_secret_key(self):
        """Should be overridden by targets to determine if a secret key needs
        to be provided for upload or not
        """
        if not self.get_upload_s3_secret_key():
            msg = (
                "Please provide the upload secret key for bucket"
                f" {self.get_upload_s3_bucket()} via endpoint"
                f" {self.get_upload_s3_endpoint()}: "
            )
            self.upload_s3_secret_key = getpass(msg)

    def prompt_for_upload_s3_bucket(self):
        """Should be overridden by targets to determine if a bucket needs to
        be provided for upload or not
        """
        if not self.upload_s3_bucket:
            if self.upload_url and self.upload_url.startswith('s3://'):
                self.upload_s3_bucket = self.upload_url[5:]
            else:
                user_input = input(_("Please provide the upload bucket: "))
                self.upload_s3_bucket = user_input.strip('/')
        return self.upload_s3_bucket

    def prompt_for_upload_s3_endpoint(self):
        """Should be overridden by targets to determine if an endpoint needs
        to be provided for upload or not
        """
        default_endpoint = self._upload_s3_endpoint
        if not self.upload_s3_endpoint:
            msg = (
                "Please provide the upload endpoint for bucket"
                f" {self.get_upload_s3_bucket()}"
                f" (default: {default_endpoint}): "
            )
            user_input = input(_(msg))
            self.upload_s3_endpoint = user_input or default_endpoint
        return self.upload_s3_endpoint

    def prompt_for_upload_user(self):
        """Should be overridden by targets to determine if a user needs to
        be provided or not
        """
        if not self.get_upload_user():
            msg = f"Please provide upload user for {self.get_upload_url()}: "
            self.upload_user = input(_(msg))

    def prompt_for_upload_password(self):
        """Should be overridden by targets to determine if a password needs to
        be provided for upload or not
        """
        if not self.get_upload_password() and (self.get_upload_user() !=
                                               self._upload_user):
            msg = ("Please provide the upload password for "
                   f"{self.get_upload_user()}: ")
            self.upload_password = getpass(msg)

    def upload_archive(self, archive):
        """
        Entry point for sos attempts to upload the generated archive to a
        target or user specified location.

        Currently there is support for HTTPS, SFTP, and FTP. HTTPS uploads are
        preferred for target-defined defaults.

        Targets that need to override uploading methods should override the
        respective upload_https(), upload_sftp(), and/or upload_ftp() methods
        and should NOT override this method.

        :param archive: The archive filepath to use for upload
        :type archive: ``str``

        In order to enable this for a target, that target needs to implement
        the following:

        Required Class Attrs

        :_upload_url:     The default location to use. Note these MUST include
                          protocol header
        :_upload_user:    Default username, if any else None
        :_upload_password: Default password, if any else None

        The following Class Attrs may optionally be overidden by the Target

        :_upload_directory:     Default FTP server directory, if any


        The following methods may be overridden by ``Target`` as needed

        `prompt_for_upload_user()`
            Determines if sos should prompt for a username or not.

        `get_upload_user()`
            Determines if the default or a different username should be used

        `get_upload_https_auth()`
            Format authentication data for HTTPS uploads

        `get_upload_url_string()`
            Print a more human-friendly string than vendor URLs
        """
        self.upload_archive_name = archive
        if not self.upload_url:
            self.upload_url = self.get_upload_url()
        if not self.upload_url:
            raise Exception("No upload destination provided by upload target"
                            " or by --upload-url")
        upload_func = self._determine_upload_type()
        self.ui_log.info(
            _(f"Attempting upload to {self.get_upload_url_string()}")
        )
        return upload_func()

    def _determine_upload_type(self):
        """Based on the url provided, determine what type of upload to attempt.

        Note that this requires users to provide a FQDN address, such as
        https://myvendor.com/api or ftp://myvendor.com instead of
        myvendor.com/api or myvendor.com
        """
        prots = {
            'ftp': self.upload_ftp,
            'sftp': self.upload_sftp,
            'https': self.upload_https,
            's3': self.upload_s3
        }
        if self.commons['cmdlineopts'].upload_protocol in prots:
            return prots[self.commons['cmdlineopts'].upload_protocol]
        if '://' not in self.upload_url:
            raise Exception("Must provide protocol in upload URL")
        prot, _ = self.upload_url.split('://')
        if prot not in prots:
            raise Exception(f"Unsupported or unrecognized protocol: {prot}")
        return prots[prot]

    def get_upload_https_auth(self, user=None, password=None):
        """Formats the user/password credentials using basic auth

        :param user: The username for upload
        :type user: ``str``

        :param password: Password for `user` to use for upload
        :type password: ``str``

        :returns: The user/password auth suitable for use in requests calls
        :rtype: ``requests.auth.HTTPBasicAuth()``
        """
        if not user:
            user = self.get_upload_user()
        if not password:
            password = self.get_upload_password()

        return requests.auth.HTTPBasicAuth(user, password)

    def get_upload_s3_access_key(self):
        """Helper function to determine if we should use the target default
        upload access key or one provided by the user

        :returns: The access_key to use for upload
        :rtype: ``str``
        """
        return (os.getenv('SOSUPLOADS3ACCESSKEY', None) or
                self.upload_s3_access_key or
                self._upload_s3_access_key)

    def get_upload_s3_endpoint(self):
        """Helper function to determine if we should use the target default
        upload endpoint or one provided by the user

        :returns: The S3 Endpoint to use for upload
        :rtype: ``str``
        """
        if not self.upload_s3_endpoint:
            self.prompt_for_upload_s3_endpoint()
        return self.upload_s3_endpoint

    def get_upload_s3_region(self):
        """Helper function to determine if we should use the target default
        upload region or one provided by the user

        :returns: The S3 region to use for upload
        :rtype: ``str``
        """
        return self.upload_s3_region or self._upload_s3_region

    def get_upload_s3_bucket(self):
        """Helper function to determine if we should use the target default
        upload bucket or one provided by the user

        :returns: The S3 bucket to use for upload
        :rtype: ``str``
        """
        if self.upload_url and self.upload_url.startswith('s3://'):
            bucket_and_prefix = self.upload_url[5:].split('/', 1)
            self.upload_s3_bucket = bucket_and_prefix[0]
            if len(bucket_and_prefix) > 1:
                self.upload_s3_object_prefix = bucket_and_prefix[1]
        if not self.upload_s3_bucket:
            self.prompt_for_upload_s3_bucket()
        return self.upload_s3_bucket or self._upload_s3_bucket

    def get_upload_s3_object_prefix(self):
        """Helper function to determine if we should use the target default
        upload object prefix or one provided by the user

        :returns: The S3 object prefix to use for upload
        :rtype: ``str``
        """
        return self.upload_s3_object_prefix or self._upload_s3_object_prefix

    def get_upload_s3_secret_key(self):
        """Helper function to determine if we should use the target default
        upload secret key or one provided by the user

        :returns: The S3 secret key to use for upload
        :rtype: ``str``
        """
        return (os.getenv('SOSUPLOADS3SECRETKEY', None) or
                self.upload_s3_secret_key or
                self._upload_s3_secret_key)

    def get_upload_url(self):
        """Helper function to determine if we should use the target default
        upload url or one provided by the user

        :returns: The URL to use for upload
        :rtype: ``str``
        """
        if not self.upload_url and (
            self.upload_s3_bucket and
            self.upload_s3_access_key and
            self.upload_s3_secret_key
        ):
            bucket = self.get_upload_s3_bucket()
            prefix = self.get_upload_s3_object_prefix()
            self._upload_url = f"s3://{bucket}/{prefix}"
        return self.upload_url or self._upload_url

    def _get_obfuscated_upload_url(self, url):
        pattern = r"([^:]+://[^:]+:)([^@]+)(@.+)"
        obfuscated_url = re.sub(pattern, r'\1********\3', url)
        return obfuscated_url

    def get_upload_url_string(self):
        """Used by upload targets to potentially change the string used to
        report upload location from the URL to a more human-friendly string
        """
        return self._get_obfuscated_upload_url(self.get_upload_url())

    def get_upload_user(self):
        """Helper function to determine if we should use the target default
        upload user or one provided by the user

        :returns: The username to use for upload
        :rtype: ``str``
        """
        return (os.getenv('SOSUPLOADUSER', None) or
                self.upload_user or
                self._upload_user)

    def get_upload_password(self):
        """Helper function to determine if we should use the target default
        upload password or one provided by the user

        A user provided password, either via option or the 'SOSUPLOADPASSWORD'
        environment variable will have precendent over any target value

        :returns: The password to use for upload
        :rtype: ``str``
        """
        return (os.getenv('SOSUPLOADPASSWORD', None) or
                self.upload_password or
                self._upload_password)

    def upload_sftp(self, user=None, password=None, user_dir=None):
        """Attempts to upload the archive to an SFTP location.

        Due to the lack of well maintained, secure, and generally widespread
        python libraries for SFTP, sos will shell-out to the system's local ssh
        installation in order to handle these uploads.

        Do not override this method with one that uses python-paramiko, as the
        upstream sos team will reject any PR that includes that dependency.
        """
        # if we somehow don't have sftp available locally, fail early
        if not is_executable('sftp'):
            raise Exception('SFTP is not locally supported')

        # soft dependency on python3-pexpect, which we need to use to control
        # sftp login since as of this writing we don't have a viable solution
        # via ssh python bindings commonly available among downstreams
        try:
            import pexpect
        except ImportError as err:
            raise Exception('SFTP upload requires python3-pexpect, which is '
                            'not currently installed') from err

        sftp_connected = False

        if not user:
            user = self.get_upload_user()
        if not password:
            password = self.get_upload_password()

        # need to strip the protocol prefix here
        sftp_url = self.get_upload_url().replace('sftp://', '')
        sftp_cmd = f"sftp -oStrictHostKeyChecking=no {user}@{sftp_url}"
        ret = pexpect.spawn(sftp_cmd, encoding='utf-8')

        sftp_expects = [
            'sftp>',
            'password:',
            'Connection refused',
            pexpect.TIMEOUT,
            pexpect.EOF
        ]

        idx = ret.expect(sftp_expects, timeout=15)

        if idx == 0:
            sftp_connected = True
        elif idx == 1:
            ret.sendline(password)
            pass_expects = [
                'sftp>',
                'Permission denied',
                pexpect.TIMEOUT,
                pexpect.EOF
            ]
            sftp_connected = ret.expect(pass_expects, timeout=10) == 0
            if not sftp_connected:
                ret.close()
                raise Exception("Incorrect username or password for "
                                f"{self.get_upload_url_string()}")
        elif idx == 2:
            raise Exception("Connection refused by "
                            f"{self.get_upload_url_string()}. Incorrect port?")
        elif idx == 3:
            raise Exception("Timeout hit trying to connect to "
                            f"{self.get_upload_url_string()}")
        elif idx == 4:
            raise Exception("Unexpected error trying to connect to sftp: "
                            f"{ret.before}")

        if not sftp_connected:
            ret.close()
            raise Exception("Unable to connect via SFTP to "
                            f"{self.get_upload_url_string()}")

        # certain implementations require file to be put in the user dir
        put_cmd = (
            f"put {self.upload_archive_name} "
            f"{f'{user_dir}/' if user_dir else ''}"
            f"{self._get_sftp_upload_name()}"
        )
        ret.sendline(put_cmd)
        put_expects = [
            '100%',
            pexpect.TIMEOUT,
            pexpect.EOF,
            'No such file or directory'
        ]

        put_success = ret.expect(put_expects, timeout=180)

        if put_success == 0:
            ret.sendline('bye')
            return True
        if put_success == 1:
            raise Exception("Timeout expired while uploading")
        if put_success == 2:
            raise Exception(f"Unknown error during upload: {ret.before}")
        if put_success == 3:
            raise Exception("Unable to write archive to destination")
        raise Exception(f"Unexpected response from server: {ret.before}")

    def _get_sftp_upload_name(self):
        """If a specific file name pattern is required by the SFTP server,
        override this method in the relevant Upload Target. Otherwise the
        archive's name on disk will be used

        :returns:       Filename as it will exist on the SFTP server
        :rtype:         ``str``
        """
        fname = self.upload_archive_name.split('/')[-1]
        if self.upload_directory:
            fname = os.path.join(self.upload_directory, fname)
        return fname

    def _upload_https_put(self, archive, verify=True):
        """If upload_https() needs to use requests.put(), use this method.

        Targets should override this method instead of the base upload_https()

        :param archive:     The open archive file object
        """
        return requests.put(self.get_upload_url(), data=archive,
                            auth=self.get_upload_https_auth(),
                            verify=verify, timeout=TIMEOUT_DEFAULT)

    def _get_upload_headers(self):
        """Define any needed headers to be passed with the POST request here
        """
        return {}

    def _upload_https_post(self, archive, verify=True):
        """If upload_https() needs to use requests.post(), use this method.

        Targets should override this method instead of the base upload_https()

        :param archive:     The open archive file object
        """
        files = {
            'file': (archive.name.split('/')[-1], archive,
                     self._get_upload_headers())
        }
        return requests.post(self.get_upload_url(), files=files,
                             auth=self.get_upload_https_auth(),
                             verify=verify, timeout=TIMEOUT_DEFAULT)

    def upload_https(self):
        """Attempts to upload the archive to an HTTPS location.

        :returns: ``True`` if upload is successful
        :rtype: ``bool``

        :raises: ``Exception`` if upload was unsuccessful
        """
        if not REQUESTS_LOADED:
            raise Exception("Unable to upload due to missing python requests "
                            "library")

        with open(self.upload_archive_name, 'rb') as arc:
            if self.commons['cmdlineopts'].upload_method == 'auto':
                method = self._upload_method
            else:
                method = self.commons['cmdlineopts'].upload_method
            verify = self.commons['cmdlineopts'].upload_no_ssl_verify is False
            if method == 'put':
                r = self._upload_https_put(arc, verify)
            else:
                r = self._upload_https_post(arc, verify)
            if r.status_code not in (200, 201):
                if r.status_code == 401:
                    raise Exception(
                        "Authentication failed: invalid user credentials"
                    )
                raise Exception(f"POST request returned {r.status_code}: "
                                f"{r.reason}")
            return True

    def upload_ftp(self, url=None, directory=None, user=None, password=None):
        """Attempts to upload the archive to either the target defined or user
        provided FTP location.

        :param url: The URL to upload to
        :type url: ``str``

        :param directory: The directory on the FTP server to write to
        :type directory: ``str`` or ``None``

        :param user: The user to authenticate with
        :type user: ``str``

        :param password: The password to use for `user`
        :type password: ``str``

        :returns: ``True`` if upload is successful
        :rtype: ``bool``

        :raises: ``Exception`` if upload in unsuccessful
        """
        import ftplib
        import socket

        if not url:
            url = self.get_upload_url()
        if url is None:
            raise Exception("no FTP server specified by upload target, "
                            "use --upload-url to specify a location")

        url = url.replace('ftp://', '')

        if not user:
            user = self.get_upload_user()

        if not password:
            password = self.get_upload_password()

        if not directory:
            directory = self.upload_directory or self._upload_directory

        try:
            session = ftplib.FTP(url, user, password, timeout=15)
            if not session:
                raise Exception("connection failed, did you set a user and "
                                "password?")
            session.cwd(directory)
        except socket.timeout as err:
            raise Exception(f"timeout hit while connecting to {url}") from err
        except socket.gaierror as err:
            raise Exception(f"unable to connect to {url}") from err
        except ftplib.error_perm as err:
            errno = str(err).split()[0]
            if errno == '503':
                raise Exception(f"could not login as '{user}'") from err
            if errno == '530':
                raise Exception(f"invalid password for user '{user}'") from err
            if errno == '550':
                raise Exception("could not set upload directory to "
                                f"{directory}") from err
            raise Exception(f"error trying to establish session: {str(err)}") \
                from err

        with open(self.upload_archive_name, 'rb') as _arcfile:
            session.storbinary(
                f"STOR {self.upload_archive_name.split('/')[-1]}", _arcfile
                )
        session.quit()
        return True

    def upload_s3(self, endpoint=None, region=None, bucket=None, prefix=None,
                  access_key=None, secret_key=None):
        """Attempts to upload the archive to an S3 bucket.

        :param endpoint: The S3 endpoint to upload to
        :type endpoint: str

        :param region: The S3 region to upload to
        :type region: str

        :param bucket: The name of the S3 bucket to upload to
        :type bucket: str

        :param prefix: The prefix for the S3 object/key
        :type prefix: str

        :param access_key: The access key for the S3 bucket
        :type access_key: str

        :param secret_key: The secret key for the S3 bucket
        :type secret_key: str

        :returns: True if upload is successful
        :rtype: bool

        :raises: Exception if upload is unsuccessful
        """
        if not BOTO3_LOADED:
            raise Exception("Unable to upload due to missing python boto3 "
                            "library")

        if not endpoint:
            endpoint = self.get_upload_s3_endpoint()
        if not region:
            region = self.get_upload_s3_region()

        if not bucket:
            bucket = self.get_upload_s3_bucket().strip('/')

        if not prefix:
            prefix = self.get_upload_s3_object_prefix()
            if prefix != '' and prefix.startswith('/'):
                prefix = prefix[1:]
            if prefix != '' and not prefix.endswith('/'):
                prefix = f'{prefix}/' if prefix else ''

        if not access_key:
            access_key = self.get_upload_s3_access_key()

        if not secret_key:
            secret_key = self.get_upload_s3_secret_key()

        s3_client = boto3.client('s3', endpoint_url=endpoint,
                                 region_name=region,
                                 aws_access_key_id=access_key,
                                 aws_secret_access_key=secret_key)

        try:
            key = prefix + self.upload_archive_name.split('/')[-1]
            s3_client.upload_file(self.upload_archive_name,
                                  bucket, key)
            return True
        except Exception as e:
            raise Exception(f"Failed to upload to S3: {str(e)}") from e

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Nis(Plugin, IndependentPlugin):

    short_desc = 'Network Information Service'
    plugin_name = 'nis'
    profiles = ('identity', 'services')

    files = ('/var/yp', '/etc/ypserv.conf')

    packages = ('ypserv',)

    def setup(self):
        self.add_copy_spec([
            "/etc/yp*.conf",
            "/var/yp/*"
        ])
        self.add_cmd_output("domainname")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2019 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Rasdaemon(Plugin, RedHatPlugin):

    short_desc = 'rasdaemon kernel trace event monitor'

    plugin_name = 'rasdaemon'
    packages = ('rasdaemon', )
    services = ('rasdaemon', )

    def setup(self):
        subcmds = [
            '--errors',
            '--guess-labels',
            '--layout',
            '--mainboard',
            '--print-labels',
            '--status',
            '--summary'
        ]
        self.add_cmd_output([f"ras-mc-ctl {sub}" for sub in subcmds])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, UbuntuPlugin


class OpenStackMasakariMonitors(Plugin, UbuntuPlugin):

    short_desc = 'OpenStack Masakari Monitors'
    plugin_name = "openstack_masakarimonitors"
    profiles = ('openstack', 'openstack_controller')

    packages = ('masakari-monitors-common', )

    services = (
        'masakari-host-monitor',
        'masakari-instance-monitor',
        'masakari-process-monitor',
    )

    config_dir = "/etc/masakarimonitors"

    def setup(self):

        self.add_copy_spec([
            self.config_dir,
        ])

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/masakarimonitors/*",
            ])
        else:
            self.add_copy_spec([
                "/var/log/masakarimonitors/*.log",
            ])

        self.add_file_tags({
            f"{self.config_dir}/masakarimonitors.conf": "masakarimonitors_conf"
        })

    def postproc(self):
        protect_keys = [".*password.*"]

        self.do_path_regex_sub(
            f"{self.config_dir}/*",
            fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)",
            r"\1*********"
        )


# vim: et ts=4 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class UnicodeTest(Plugin, IndependentPlugin):
    """Fake plugin to test the handling of a file with embedded unicode
    """

    plugin_name = 'unicode_test'
    short_desc = 'Fake plugin to test unicode file handling'

    def setup(self):
        self.add_copy_spec('/tmp/sos-test-unicode.txt')



# Copyright (C) 2009 Red Hat, Inc., Joey Boggs <jboggs@redhat.com>
# Copyright (C) 2012 Rackspace US, Inc.,
#                    Justin Shepherd <jshepher@rackspace.com>
# Copyright (C) 2013 Red Hat, Inc., Jeremy Agee <jagee@redhat.com>
# Copyright (C) 2015 Red Hat, Inc., Abhijeet Kasurde <akasurde@redhat.com>
# Copyright (C) 2017 Red Hat, Inc., Martin Schuppert <mschuppert@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
import re
from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class OpenStackNova(Plugin):

    short_desc = 'OpenStack Nova'
    plugin_name = "openstack_nova"
    profiles = ('openstack', 'openstack_controller',
                'openstack_compute', 'openstack_edpm')
    containers = ('.*nova_api',)

    var_puppet_gen = "/var/lib/config-data/puppet-generated/nova"
    service_name = "openstack-nova-api.service"
    apachepkg = None

    def setup(self):

        # collect commands output only if the openstack-nova-api service
        # is running
        in_container = self.container_exists('.*nova_api')

        if self.is_service_running(self.service_name) or in_container:
            nova_config = ""
            # if containerized we need to pass the config to the cont.
            if in_container:
                nova_config = "--config-dir " + self.var_puppet_gen + \
                                "/etc/nova/"

            self.add_cmd_output(
                "nova-manage " + nova_config + " db version",
                suggest_filename="nova-manage_db_version"
            )
            self.add_cmd_output(
                "nova-manage " + nova_config + " fixed list",
                suggest_filename="nova-manage_fixed_list"
            )
            self.add_cmd_output(
                "nova-manage " + nova_config + " floating list",
                suggest_filename="nova-manage_floating_list"
            )
            self.add_cmd_output(
                "nova-status " + nova_config + " upgrade check",
                suggest_filename="nova-status_upgrade_check"
            )

            vars_all = [p in os.environ for p in [
                        'OS_USERNAME', 'OS_PASSWORD']]

            vars_any = [p in os.environ for p in [
                        'OS_TENANT_NAME', 'OS_PROJECT_NAME']]

            if not (all(vars_all) and any(vars_any)):
                self.soslog.warning("Not all environment variables set. "
                                    "Source the environment file for the user "
                                    "intended to connect to the OpenStack "
                                    "environment.")
            else:
                self.add_cmd_output("openstack compute service list")
                self.add_cmd_output("openstack flavor list --long")
                self.add_cmd_output("openstack compute agent list")
                self.add_cmd_output("nova version-list")
                self.add_cmd_output("openstack hypervisor list")
                self.add_cmd_output("openstack quota show")
                self.add_cmd_output("openstack hypervisor stats show")

                res = self.collect_cmd_output(
                    "openstack server list --all-projects"
                )

                # get details for each nova instance
                if res['status'] == 0:
                    nova_instances = res['output']
                    for instance in nova_instances.splitlines()[3:-1]:
                        instance = instance.split()[1]
                        cmd = f"openstack server show {instance}"
                        self.add_cmd_output(cmd)

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/nova/",
                f"/var/log/{self.apachepkg}*/nova*",
                "/var/lib/nova/instances/*/console.log",
            ])
        else:
            novadir = '/var/log/nova/'
            novalogs = [
                "nova-api.log*",
                "nova-compute.log*",
                "nova-conductor.log*",
                "nova-metadata-api.log*",
                "nova-manage.log*",
                "nova-placement-api.log*",
                "nova-scheduler.log*"
            ]
            for novalog in novalogs:
                self.add_copy_spec(self.path_join(novadir, novalog))
            self.add_copy_spec([
                f"/var/log/{self.apachepkg}*/nova*.log",
                f"/var/log/{self.apachepkg}*/placement*.log",
            ])

        npaths = ['', '_libvirt', '_metadata', '_placement']
        syspaths = [
            '/etc/nova/',
            '/etc/my.cnf.d/tripleo.cnf',
            '/etc/httpd/conf/',
            '/etc/httpd/conf.d/',
            '/etc/httpd/conf.modules.d/*.conf'
        ]
        # excludes httpd'ish specs in the libvirt path
        specs = [
            "/etc/nova/",
            "authorized_keys",
            self.var_puppet_gen + "/../memcached/etc/sysconfig/memcached",
            self.var_puppet_gen + "/var/spool/cron/nova",
            self.var_puppet_gen + "_libvirt/etc/libvirt/",
            self.var_puppet_gen + "_libvirt/etc/nova/migration/",
            self.var_puppet_gen + "_libvirt/var/lib/nova/.ssh/config"
        ] + list(
            filter(re.compile('^((?!libvirt.+httpd).)*$').match,
                   [f'{self.var_puppet_gen}{p}{s}'
                    for p in npaths for s in syspaths
                    ]))
        self.add_copy_spec(specs)

    def apply_regex_sub(self, regexp, subst):
        """ Apply regex substitution """
        self.do_path_regex_sub("/etc/nova/*", regexp, subst)
        for npath in ['', '_libvirt', '_metadata', '_placement']:
            self.do_path_regex_sub(
                f"{self.var_puppet_gen}{npath}/etc/nova/*",
                regexp, subst)

    def postproc(self):
        protect_keys = [
            "ldap_dns_password", "neutron_admin_password", "rabbit_password",
            "qpid_password", "powervm_mgr_passwd", "virtual_power_host_pass",
            "xenapi_connection_password", "password", "host_password",
            "vnc_password", "admin_password", "connection_password",
            "memcache_secret_key", "s3_secret_key",
            "metadata_proxy_shared_secret", "fixed_key", "transport_url",
            "rbd_secret_uuid"
        ]
        connection_keys = ["connection", "sql_connection"]

        join_con_keys = "|".join(connection_keys)

        self.apply_regex_sub(
            fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)",
            r"\1*********"
        )
        self.apply_regex_sub(
            fr"(^\s*({join_con_keys})\s*=\s*(.*)://(\w*):)(.*)(@(.*))",
            r"\1*********\6"
        )


class DebianNova(OpenStackNova, DebianPlugin, UbuntuPlugin):

    apachepkg = "apache2"
    nova = False
    packages = (
        'nova-api-ec2',
        'nova-api-metadata',
        'nova-api-os-compute',
        'nova-api-os-volume',
        'nova-common',
        'nova-compute',
        'nova-compute-kvm',
        'nova-compute-lxc',
        'nova-compute-qemu',
        'nova-compute-uml',
        'nova-compute-xcp',
        'nova-compute-xen',
        'nova-xcp-plugins',
        'nova-consoleauth',
        'nova-network',
        'nova-scheduler',
        'nova-volume',
        'novnc',
        'python-nova',
        'python-novnc',
        'python3-nova',
    )
    service_name = "nova-api.service"

    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/sudoers.d/nova_sudoers",
            "/usr/share/polkit-1/rules.d/60-libvirt.rules",
        ])


class RedHatNova(OpenStackNova, RedHatPlugin):

    apachepkg = "httpd"
    nova = False
    packages = ('openstack-selinux',)

    def setup(self):
        super().setup()
        self.add_copy_spec([
            "/etc/logrotate.d/openstack-nova",
            "/etc/polkit-1/localauthority/50-local.d/50-nova.pkla",
            "/etc/sudoers.d/nova",
            "/etc/security/limits.d/91-nova.conf",
            "/etc/sysconfig/openstack-nova-novncproxy",
            "/var/lib/openstack/config/nova",
            "/var/lib/openstack/containers/nova*.json"
        ])

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/httpd/placement*",
                "/var/log/containers/nova/*"
            ])
        else:
            self.add_copy_spec([
                "/var/log/httpd/placement*.log",
                "/var/log/containers/nova/*.log"
            ])

        self.add_forbidden_path([
            "/var/lib/openstack/config/nova/ssh-privatekey"
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2014 Red Hat, Inc., Peter Portante <peter.portante@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Tuned(Plugin, RedHatPlugin):

    short_desc = 'Tuned system tuning daemon'
    packages = ('tuned',)
    profiles = ('system', 'performance')
    plugin_name = 'tuned'

    def setup(self):
        self.add_cmd_output("tuned-adm list",
                            tags="tuned_adm")
        self.add_cmd_output([
            "tuned-adm active",
            "tuned-adm recommend",
            "tuned-adm verify"
        ])

        self.add_copy_spec("/etc/tuned.conf",
                           tags="tuned_conf")
        self.add_copy_spec([
            "/etc/tuned",
            "/etc/tune-profiles",
            "/usr/lib/tuned",
            "/var/log/tuned/tuned.log"
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2023 Red Hat, Inc., Roberto Alfieri <ralfieri@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class OpenStackEDPM(Plugin, RedHatPlugin):

    short_desc = 'Installation information from OpenStack EDPM deployment'

    plugin_name = 'openstack_edpm'
    profiles = ('openstack', 'openstack_edpm')
    services = 'edpm-container-shutdown'
    edpm_log_paths = []

    def setup(self):
        # Notes: recursion is max 2 for edpm-config
        # Those directories are present on all OpenStack nodes
        self.edpm_log_paths = [
            '/etc/os-net-config/',
            '/var/lib/config-data/',
            '/var/lib/edpm-config/',
        ]
        self.add_copy_spec(self.edpm_log_paths)

    def postproc(self):
        # Ensures we do not leak passwords from the edpm related locations
        # Other locations don't have sensitive data.
        regexp = r'(".*(key|password|pass|secret|database_connection))' \
                 r'([":\s]+)(.*[^"])([",]+)'
        for path in self.edpm_log_paths:
            self.do_path_regex_sub(path, regexp, r'\1\3*********\5')

# vim: set et ts=4 sw=4 :



# Copyright (C) 2015 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class TargetCli(Plugin, IndependentPlugin):

    short_desc = 'TargetCLI TCM/LIO configuration'
    packages = ('targetcli', 'python-rtslib')
    profiles = ('storage', )
    plugin_name = 'targetcli'

    def setup(self):
        self.add_cmd_output([
            "targetcli ls",
            "targetcli status",
        ])
        sys_conf_dir = '/sys/kernel/config/target'
        self.add_forbidden_path([
            self.path_join(sys_conf_dir, '**/password*'),
        ])
        self.add_service_status("target")
        self.add_journal(units="targetcli")
        self.add_copy_spec(sys_conf_dir)
        self.add_copy_spec("/etc/target")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class OSTree(Plugin, IndependentPlugin):

    short_desc = 'OSTree'

    plugin_name = 'ostree'
    profiles = ('system', 'sysmgmt', 'packagemanager')
    files = ('/ostree',)
    services = ('ostree-finalize-staged', 'ostree-boot-complete')
    option_list = [
        PluginOpt('fsck', default=False, desc='collect ostree fsck')
    ]

    def setup(self):
        self.add_copy_spec("/ostree/repo/config")
        self.add_cmd_output([
            "ostree admin status",
            "ostree admin config-diff",
            "ostree refs",
        ])
        if self.get_option("fsck"):
            self.add_cmd_output("ostree fsck")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2020 Red Hat, Inc., Nitin Yewale <nyewale@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class NvmetCli(Plugin, IndependentPlugin):

    short_desc = 'Collect config and system information for nvmetcli'

    packages = ('nvmetcli', )
    profiles = ('storage', )
    plugin_name = 'nvmetcli'

    def setup(self):
        self.add_cmd_output([
            "nvmetcli ls"
        ])
        self.add_journal(units=["nvme", "nvmet", "nvmet_rdma"])
        self.add_copy_spec([
            "/sys/kernel/config/nvmet",
            "/etc/nvmet/config.json",
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Curtin(Plugin, IndependentPlugin):

    short_desc = 'Curt Installer'

    plugin_name = 'curtin'
    profiles = ('boot',)
    files = ('/root/curtin-install-cfg.yaml', )

    def setup(self):
        self.add_copy_spec([
            '/root/curtin-install.log',
            '/root/curtin-install-cfg.yaml',
            '/etc/default/grub.d/50-curtin-settings.cfg',
            '/etc/apt/apt.conf.d/90curtin-aptproxy',
            '/etc/apt/sources.list.curtin.old',
            '/etc/cloud/cloud.cfg.d/50-curtin-networking.cfg',
            '/etc/cloud/cloud.cfg.d/curtin-preserve-sources.cfg',
        ])

    def postproc(self):
        protect_keys = [
            "oauth_consumer_key",
            "oauth_token_key",
            "token_key",
            "token_secret",
            "consumer_key",
        ]
        curtin_files = [
            "/root/curtin-install-cfg.yaml",
            "/root/curtin-install.log",
        ]

        match_exp_multil = fr"({'|'.join(protect_keys)})\s*(:|=)(\S*\n.*?\\n)"
        match_exp = fr"({'|'.join(protect_keys)})\s*(:|=)\s*[a-zA-Z0-9]*"

        for file in curtin_files:
            self.do_file_sub(
                file, match_exp_multil,
                r"\1\2*********"
            )
            self.do_file_sub(
                file, match_exp,
                r"\1\2*********"
            )

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import platform
from sos.report.plugins import (Plugin, PluginOpt, RedHatPlugin, DebianPlugin,
                                UbuntuPlugin, CosPlugin, AzurePlugin)


class KDump(Plugin):

    short_desc = 'Kdump crash dumps'

    plugin_name = "kdump"
    profiles = ('system', 'debug')

    def setup(self):
        self.add_copy_spec([
            "/proc/cmdline",
            "/etc/sysconfig/kdump",
            "/proc/sys/kernel/panic",
            "/proc/sys/kernel/panic_on_oops",
            "/sys/kernel/kexec_loaded",
            "/sys/kernel/fadump",
            "/sys/kernel/fadump_enabled",
            "/sys/kernel/fadump_registered",
            "/sys/kernel/kexec_crash_size"
        ])

        self.add_copy_spec("/sys/kernel/kexec_crash_loaded",
                           tags="kexec_crash_loaded")


class RedHatKDump(KDump, RedHatPlugin):

    files = ('/etc/kdump.conf',)
    packages = ('kexec-tools',)

    option_list = [
        PluginOpt("get-vm-core", default=False, val_type=bool,
                  desc="collect vm core")
    ]

    def fstab_parse_fs(self, device):
        """ Parse /etc/fstab file """
        fstab = self.path_join('/etc/fstab')
        with open(fstab, 'r', encoding='UTF-8') as file:
            for line in file:
                if line.startswith((device)):
                    return line.split()[1].rstrip('/')
        return ""

    def read_kdump_conffile(self):
        """ Parse /etc/kdump file """
        fsys = ""
        path = "/var/crash"

        kdump = '/etc/kdump.conf'
        with open(kdump, 'r', encoding='UTF-8') as file:
            for line in file:
                if line.startswith("path"):
                    path = line.split()[1]
                elif line.startswith(("ext2", "ext3", "ext4", "xfs")):
                    device = line.split()[1]
                    fsys = self.fstab_parse_fs(device)
        return fsys + path

    def setup(self):
        super().setup()

        initramfs_img = "/boot/initramfs-" + platform.release() \
                        + "kdump.img"
        if self.path_exists(initramfs_img):
            self.add_cmd_output(f"lsinitrd {initramfs_img}")

        self.add_copy_spec([
            "/etc/kdump.conf",
            "/etc/udev/rules.d/*kexec.rules",
            "/usr/lib/udev/rules.d/*kexec.rules",
            "/var/crash/*/kexec-dmesg.log",
            "/var/log/kdump.log",
            "/var/crash/*/vmcore-creation.status",
        ])
        self.add_copy_spec("/var/crash/*/vmcore-dmesg.txt",
                           tags="vmcore_dmesg")
        try:
            path = self.read_kdump_conffile()
        except Exception:  # pylint: disable=broad-except
            # set no filesystem and default path
            path = "/var/crash"

        self.add_dir_listing(path, recursive=True)
        self.add_copy_spec(f"{path}/*/vmcore-dmesg.txt")
        self.add_copy_spec(f"{path}/*/kexec-dmesg.log")
        self.add_copy_spec(f"{path}/*/vmcore-creation.status")

        # collect the latest vmcore created in the last 24hrs <= 2GB
        if self.get_option("get-vm-core"):
            self.add_copy_spec(f"{path}/*/vmcore", sizelimit=2048, maxage=24)

        # collect status via kdumpctl
        self.add_cmd_output([
            "kdumpctl status",
            "kdumpctl estimate",
        ])


class DebianKDump(KDump, DebianPlugin, UbuntuPlugin):

    files = ('/etc/default/kdump-tools',)
    packages = ('kdump-tools',)

    def setup(self):
        super().setup()

        initramfs_img = "/var/lib/kdump/initrd.img-" + platform.release()
        if self.path_exists(initramfs_img):
            self.add_cmd_output(f"lsinitramfs -l {initramfs_img}")

        self.add_cmd_output("kdump-config show")

        self.add_copy_spec([
            "/etc/default/kdump-tools"
        ])


class CosKDump(KDump, CosPlugin):

    option_list = [
        PluginOpt(name="collect-kdumps", default=False,
                  desc="Collect existing kdump files"),
    ]

    def setup(self):
        super().setup()
        self.add_dir_listing('/var/kdump*', recursive=True)
        if self.get_option("collect-kdumps"):
            self.add_copy_spec(["/var/kdump-*"])


class AzureKDump(KDump, AzurePlugin):

    files = ('/etc/kdump.conf',)
    packages = ('kexec-tools',)

    option_list = [
        PluginOpt("get-vm-core", default=False, val_type=bool,
                  desc="collect vm core")
    ]

    def read_kdump_conffile(self):
        """ Parse /etc/kdump file """
        path = "/var/crash"

        kdump = '/etc/kdump.conf'
        with open(kdump, 'r', encoding='UTF-8') as file:
            for line in file:
                if line.startswith("path"):
                    path = line.split()[1]

        return path

    def setup(self):
        super().setup()

        self.add_copy_spec([
            "/etc/kdump.conf",
            "/usr/lib/udev/rules.d/*kexec.rules"
        ])

        try:
            path = self.read_kdump_conffile()
        except Exception:  # pylint: disable=broad-except
            # set no filesystem and default path
            path = "/var/crash"

        self.add_dir_listing(path, recursive=True)
        self.add_copy_spec(f"{path}/*/vmcore-dmesg.txt")
        self.add_copy_spec(f"{path}/*/kexec-dmesg.log")
        self.add_copy_spec(f"{path}/*/vmcore-creation.status")

        # collect the latest vmcore created in the last 24hrs <= 2GB
        if self.get_option("get-vm-core"):
            self.add_copy_spec(f"{path}/*/vmcore", sizelimit=2048, maxage=24)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Red Hat, Inc., Pavel Moravec <pmoravec@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Sudo(Plugin, IndependentPlugin):

    short_desc = 'Sudo command execution'
    plugin_name = 'sudo'
    profiles = ('identity', 'system')
    packages = ('sudo',)

    def setup(self):
        self.add_copy_spec("/etc/sudo*")

        config_file = "/etc/sudo.conf"
        log_files = ['/var/log/sudo_debug', '/var/log/sudoers_debug']
        try:
            with open(config_file, 'r', encoding='UTF-8') as cfile:
                for line in cfile:
                    if line.startswith('Debug'):
                        log_files.append(line.split()[2])
        except IOError as error:
            self._log_error(f'Could not open conf file {config_file}: '
                            f'{error}')

        if not self.get_option('all_logs'):
            self.add_copy_spec(log_files)
        else:
            self.add_copy_spec([f"{log}*" for log in log_files])

    def postproc(self):
        regexp = r"(\s*bindpw\s*)\S+"
        self.do_file_sub("/etc/sudo-ldap.conf", regexp, r"\1********")


# vim: set et ts=4 sw=4 :



# Copyright (c) 2024 Pavel Moravec <pmoravec@redhat.com>
# Copyright (c) 2024 Lucas Benedito <lbenedit@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class AAPControllerPlugin(Plugin, RedHatPlugin):
    short_desc = 'AAP Automation Controller plugin'
    plugin_name = 'aap_controller'
    profiles = ('sysmgmt', 'ansible',)

    packages = ('automation-controller-venv-tower',
                'automation-controller-server',
                'automation-controller-ui',
                'automation-controller')
    commands = ('awx-manage',)
    services = ('automation-controller',)

    def setup(self):
        self.add_copy_spec([
            "/etc/tower/",
            "/etc/supervisord.conf",
            "/etc/supervisord.d/*",
            "/var/log/tower",
            "/var/log/nginx/automationcontroller.access.log*",
            "/var/log/nginx/automationcontroller.error.log*",
            "/var/log/supervisor",
            "/var/log/unattended-upgrades",
        ])

        self.add_forbidden_path([
            "/etc/tower/SECRET_KEY",
            "/etc/tower/*.key",
            "/etc/tower/*.cert",
            "/var/log/tower/profile",
        ])

        self.add_cmd_output([
            "awx-manage --version",
            "awx-manage list_instances",
            "awx-manage run_dispatcher --status",
            "awx-manage run_callback_receiver --status",
            "awx-manage check_license --data",
            "awx-manage run_wsbroadcast --status",
            "awx-manage run_wsrelay --status",
            "supervisorctl status",
            "/var/lib/awx/venv/awx/bin/pip freeze",
            "/var/lib/awx/venv/awx/bin/pip freeze -l",
            "/var/lib/awx/venv/ansible/bin/pip freeze",
            "/var/lib/awx/venv/ansible/bin/pip freeze -l",
            "umask -p",
        ])

        self.add_dir_listing([
            '/var/lib/awx',
            '/var/lib/awx/venv',
            '/etc/tower',
        ])
        self.add_dir_listing('/var/lib/awx', tree=True)

    def postproc(self):
        # remove database password
        jreg = r"(\s*\'PASSWORD\'\s*:(\s))(?:\"){1,}(.+)(?:\"){1,}"
        repl = r"\1********"
        self.do_path_regex_sub("/etc/tower/conf.d/postgres.py", jreg, repl)

        # remove email password
        jreg = r"(EMAIL_HOST_PASSWORD\s*=)\'(.+)\'"
        repl = r"\1********"
        self.do_path_regex_sub("/etc/tower/settings.py", jreg, repl)

        # remove email password (if customized)
        jreg = r"(EMAIL_HOST_PASSWORD\s*=)\'(.+)\'"
        repl = r"\1********"
        self.do_path_regex_sub("/etc/tower/conf.d/custom.py", jreg, repl)

        # remove websocket secret
        jreg = r"(BROADCAST_WEBSOCKET_SECRET\s*=\s*)\"(.+)\""
        repl = r"\1********"
        self.do_path_regex_sub("/etc/tower/conf.d/channels.py", jreg, repl)


# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Dbus(Plugin, IndependentPlugin):

    short_desc = 'D-Bus message bus'

    plugin_name = "dbus"
    profiles = ('system',)
    packages = ('dbus',)

    def setup(self):
        self.add_copy_spec([
            "/etc/dbus-1",
            "/var/lib/dbus/machine-id"
        ])

        self.add_cmd_output([
            "busctl list --no-pager",
            "busctl status"
        ])

        self.add_env_var('DBUS_SESSION_BUS_ADDRESS')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re
from datetime import datetime, timedelta
from sos.report.plugins import (Plugin, RedHatPlugin, DebianPlugin,
                                UbuntuPlugin, PluginOpt)
from sos.utilities import sos_parse_version


class Pacemaker(Plugin):

    short_desc = 'Pacemaker high-availability cluster resource manager'

    plugin_name = "pacemaker"
    profiles = ("cluster", )
    packages = (
        "pacemaker",
        "pacemaker-remote",
    )

    option_list = [
        PluginOpt('crm-from', default='', val_type=str,
                  desc='specfiy the start time for crm_report'),
        PluginOpt('crm-scrub', default=True,
                  desc='enable crm_report password scrubbing')
    ]

    envfile = ""

    def setup_crm_mon(self):
        """ Get cluster summary """
        self.add_cmd_output("crm_mon -1 -A -n -r -t")

    def setup_crm_shell(self):
        """ Get cluster status and configuration """
        self.add_cmd_output([
            "crm status",
            "crm configure show",
        ])

    def setup_pcs(self):
        """ Get pacemaker/corosync configuration """
        pcs_pkg = self.policy.package_manager.pkg_by_name('pcs')
        if pcs_pkg is None:
            return

        self.add_copy_spec("/var/log/pcsd/pcsd.log")
        self.add_cmd_output([
            "pcs stonith sbd status --full",
            "pcs stonith sbd watchdog list",
            "pcs stonith history show",
        ])

        pcs_version = '.'.join(pcs_pkg['version'])
        if sos_parse_version(pcs_version) > sos_parse_version('0.10.8'):
            self.add_cmd_output("pcs property config --all")
        else:
            self.add_cmd_output("pcs property list --all")

        self.add_cmd_output("pcs config", tags="pcs_config")
        self.add_cmd_output("pcs quorum status", tags="pcs_quorum_status")
        self.add_cmd_output("pcs status --full", tags="pcs_status")

    def postproc_crm_shell(self):
        """ Clear password """
        self.do_cmd_output_sub(
            "crm configure show",
            r"passw([^\s=]*)=\S+",
            r"passw\1=********"
        )

    def postproc_pcs(self):
        """ Clear password """
        self.do_cmd_output_sub(
            "pcs config",
            r"passw([^\s=]*)=\S+",
            r"passw\1=********"
        )

    def setup(self):
        self.add_copy_spec([
            # Pacemaker 2.x default log locations
            "/var/log/pacemaker/pacemaker.log*",
            "/var/log/pacemaker/bundles/*/",
            "/var/log/pacemaker/pengine*",

            # Pacemaker 1.x default log locations
            "/var/log/pacemaker.log",
            "/var/log/pacemaker/bundles/*/",

            # Common user-specified locations
            "/var/log/cluster/pacemaker.log*",
            "/var/log/cluster/bundles/*/",
        ])

        self.setup_crm_mon()

        # crm_report needs to be given a --from "YYYY-MM-DD HH:MM:SS" start
        # time in order to collect data.
        crm_from = (datetime.today() -
                    timedelta(hours=72)).strftime("%Y-%m-%d %H:%m:%S")
        if self.get_option("crm-from"):
            if re.match(r'\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}',
                        str(self.get_option("crm-from"))):
                crm_from = self.get_option("crm-from")
            else:
                self._log_error(
                    f"crm_from parameter '{self.get_option('crm-from')}' is "
                    "not a valid date: using default")

        crm_dest = self.get_cmd_output_path(name="crm_report", make=False)
        if self.get_option("crm-scrub"):
            crm_scrub = '-p "passw.*"'
        else:
            crm_scrub = ""
            self._log_warn("scrubbing of crm passwords has been disabled:")
            self._log_warn("data collected by crm_report may contain"
                           " sensitive values.")
        self.add_cmd_output(f'crm_report --sos-mode {crm_scrub} -S -d '
                            f' --dest {crm_dest} --from "{crm_from}"',
                            chroot=self.tmp_in_sysroot())

        # collect user-defined logfiles, matching a shell-style syntax:
        #   PCMK_logfile=filename
        # specified in the pacemaker start-up environment file.
        pattern = r'^\s*PCMK_logfile=[\'\"]?(\S+)[\'\"]?\s*(\s#.*)?$'
        if self.path_isfile(self.envfile):
            self.add_copy_spec(self.envfile)
            with open(self.envfile, 'r', encoding='UTF-8') as file:
                for line in file:
                    if re.match(pattern, line):
                        # remove trailing and leading quote marks, in case the
                        # line is e.g. PCMK_logfile="/var/log/pacemaker.log"
                        logfile = re.search(pattern, line).group(1)
                        for regexp in [r'^"', r'"$', r'^\'', r'\'$']:
                            logfile = re.sub(regexp, '', logfile)
                        self.add_copy_spec(logfile)


class DebianPacemaker(Pacemaker, DebianPlugin, UbuntuPlugin):
    def setup(self):
        self.envfile = self.path_join("/etc/default/pacemaker")
        self.setup_crm_shell()
        self.setup_pcs()
        super().setup()

    def postproc(self):
        self.postproc_crm_shell()
        self.postproc_pcs()


class RedHatPacemaker(Pacemaker, RedHatPlugin):
    def setup(self):
        self.envfile = self.path_join("/etc/sysconfig/pacemaker")
        self.setup_pcs()
        self.add_copy_spec("/etc/sysconfig/sbd")
        super().setup()

    def postproc(self):
        self.postproc_pcs()

# vim: et ts=4 sw=4



# Copyright (C) 2013 Red Hat, Inc., Eoghan Lynn <eglynn@redhat.com>
# Copyright (C) 2012 Rackspace US, Inc.
#               2012 Justin Shepherd <jshepher@rackspace.com>
# Copyright (C) 2009 Red Hat, Inc.
#               2009 Joey Boggs <jboggs@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class OpenStackCeilometer(Plugin):

    short_desc = 'Openstack Ceilometer'
    plugin_name = "openstack_ceilometer"
    profiles = ('openstack', 'openstack_controller', 'openstack_compute')
    var_puppet_gen = "/var/lib/config-data/puppet-generated/ceilometer"

    def setup(self):
        # Ceilometer
        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/ceilometer/*",
            ])
        else:
            self.add_copy_spec([
                "/var/log/ceilometer/*.log",
            ])
        self.add_copy_spec([
            "/etc/ceilometer/*",
            self.var_puppet_gen + "/etc/ceilometer/*"
        ])

        self.add_file_tags({
            "/var/log/ceilometer/central.log":
                "ceilometer_central_log"
        })

    def apply_regex_sub(self, regexp, subst):
        """ Apply regex substitution """
        self.do_path_regex_sub("/etc/ceilometer/*", regexp, subst)
        self.do_path_regex_sub(
            self.var_puppet_gen + "/etc/ceilometer/*",
            regexp, subst
        )

    def postproc(self):
        protect_keys = [
            "admin_password", "connection_password", "host_password",
            "memcache_secret_key", "os_password", "password", "qpid_password",
            "rabbit_password", "readonly_user_password", "secret_key",
            "ssl_key_password", "telemetry_secret", "metering_secret"
        ]
        connection_keys = ["connection", "backend_url", "transport_url"]

        join_con_keys = "|".join(connection_keys)

        self.apply_regex_sub(
            fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)",
            r"\1*********"
        )
        self.apply_regex_sub(
            fr"(^\s*({join_con_keys})\s*=\s*(.*)://(\w*):)(.*)(@(.*))",
            r"\1*********\6"
        )


class DebianCeilometer(OpenStackCeilometer, DebianPlugin,
                       UbuntuPlugin):

    packages = (
        'ceilometer-api',
        'ceilometer-agent-central',
        'ceilometer-agent-compute',
        'ceilometer-agent-notification',
        'ceilometer-collector',
        'ceilometer-common',
        'python-ceilometer',
        'python3-ceilometer',
    )


class RedHatCeilometer(OpenStackCeilometer, RedHatPlugin):

    packages = ('openstack-selinux',)

    def setup(self):
        super().setup()
        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/containers/ceilometer/*",
            ])
        else:
            self.add_copy_spec([
                "/var/log/containers/ceilometer/*.log",
            ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Grub(Plugin, IndependentPlugin):

    short_desc = 'GRUB bootloader'

    plugin_name = 'grub'
    profiles = ('boot',)
    packages = ('grub',)

    def setup(self):
        self.add_copy_spec([
            "/boot/efi/EFI/*/grub.conf",
            "/boot/grub/grub.conf",
            "/boot/grub/device.map",
            "/etc/grub.conf",
            "/etc/grub.d"
        ])

    def postproc(self):
        self.do_path_regex_sub(
            r".*\/grub.conf",
            r"(password\s*)(--md5\s*|\s*)(.*)",
            r"\1\2********"
        )

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class KataContainers(Plugin, IndependentPlugin):

    short_desc = 'Kata Containers configuration'

    plugin_name = 'kata_containers'
    profiles = ('system', 'virt', 'container')
    packages = ('kata-containers',)

    def setup(self):

        self.add_cmd_output('kata-runtime kata-env')

        config_files = set()

        # start with the default file locations
        config_files.add('/etc/kata-containers/configuration.toml')
        config_files.add(
                '/usr/share/defaults/kata-containers/configuration.toml')

        # obtain a list of config files by asking the runtime
        cmd = 'kata-runtime --kata-show-default-config-paths'
        configs = self.collect_cmd_output(cmd)
        if configs and configs['status']:
            for config in configs['output'].splitlines():
                if config != "":
                    config_files.add(config)

            self.add_copy_spec(config_files)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2014 Red Hat, Inc. Bryn M. Reeves <bmr@redhat.com>
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Teamd(Plugin, IndependentPlugin):

    short_desc = 'Network Interface Teaming'
    plugin_name = 'teamd'
    profiles = ('network', 'hardware', )

    packages = ('teamd',)

    def setup(self):
        self.add_copy_spec([
            "/etc/dbus-1/system.d/teamd.conf",
            "/usr/lib/systemd/system/teamd@.service"
        ])

        self.add_device_cmd("teamdctl %(dev)s config dump", devices="team",
                            tags="teamdctl_config_dump")
        self.add_device_cmd("teamdctl %(dev)s state dump", devices="team",
                            tags="teamdctl_state_dump")

        self.add_device_cmd([
            "teamdctl %(dev)s state",
            "teamnl %(dev)s option",
            "teamnl %(dev)s ports"
        ], devices='team')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class UltraPath(Plugin, RedHatPlugin):

    short_desc = 'HUAWEI UltraPath'

    plugin_name = 'ultrapath'
    profiles = ('storage', 'hardware')
    packages = ('UltraPath',)
    kernel_mods = ('nxup', 'nxupext_a')

    def setup(self):
        """ Huawei UltraPath specific information - commands
        """
        self.add_cmd_output([
            "upadm show version",
            "upadm show connectarray",
            "upadm show option",
            "upadm show upconfig",
            "upadm show diskarray",
            "upadmin show vlun",
        ])

        result = self.collect_cmd_output('upadm show path')
        if result['status'] == 0:
            for line in result['output'].splitlines():
                if line.startswith("Array ID :"):
                    self.add_cmd_output(f"upadm show lun "
                                        f"array={line.split(':')[1].strip()}")

# vim: set et ts=4 sw=4 :



# Copyright (C) 2016 Red Hat, Inc. Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin


class IpmiTool(Plugin, RedHatPlugin, DebianPlugin):

    short_desc = 'IpmiTool hardware information'

    plugin_name = 'ipmitool'
    profiles = ('hardware', 'system', )

    packages = ('ipmitool',)

    def setup(self):
        cmd = "ipmitool"
        result = self.collect_cmd_output("ipmitool -I usb mc info")
        if result['status'] == 0:
            cmd += " -I usb"

        for subcmd in ['channel info', 'channel getaccess', 'lan print']:
            for channel in [1, 3]:
                self.add_cmd_output(f"{cmd} {subcmd} {channel}")

        # raw 0x30 0x65: Get HDD drive Fault LED State
        # raw 0x30 0xb0: Get LED Status

        self.add_cmd_output([
            f"{cmd} raw 0x30 0x65",
            f"{cmd} raw 0x30 0xb0",
            f"{cmd} sel info",
            f"{cmd} sel elist",
            f"{cmd} sel list -v",
            f"{cmd} sensor list",
            f"{cmd} chassis status",
            f"{cmd} lan print",
            f"{cmd} fru print",
            f"{cmd} mc info",
            f"{cmd} sdr info",
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2021 Red Hat, Inc., Nadia Pinaeva <npinaeva@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import json

from shlex import quote
from sos.policies.runtimes import ContainerRuntime
from sos.utilities import sos_get_command_output


class CrioContainerRuntime(ContainerRuntime):
    """Runtime class to use for systems running crio"""

    name = 'crio'
    binary = 'crictl'

    def check_can_copy(self):
        return False

    def get_containers(self, get_all=False):
        """Get a list of containers present on the system.

        :param get_all: If set, include stopped containers as well
        :type get_all: ``bool``
        """
        containers = []
        _cmd = f"{self.binary} ps {'-a' if get_all else ''} -o json"
        if self.active:
            out = sos_get_command_output(_cmd, chroot=self.policy.sysroot)
            if out["status"] == 0:
                out_json = json.loads(out["output"])
                for container in out_json["containers"]:
                    # takes the form (container_id, container_name)
                    containers.append(
                        (container["id"], container["metadata"]["name"]))
        return containers

    def get_images(self):
        """Get a list of images present on the system

        :returns: A list of 2-tuples containing (image_name, image_id)
        :rtype: ``list``
        """
        images = []
        if self.active:
            out = sos_get_command_output(f"{self.binary} images -o json",
                                         chroot=self.policy.sysroot)
            if out['status'] == 0:
                out_json = json.loads(out["output"])
                for image in out_json["images"]:
                    # takes the form (repository:tag, image_id)
                    if len(image["repoTags"]) > 0:
                        for repo_tag in image["repoTags"]:
                            images.append((repo_tag, image["id"]))
                    else:
                        if len(image["repoDigests"]) == 0:
                            image_name = "<none>"
                        else:
                            image_name = image["repoDigests"][0].split("@")[0]
                        images.append((image_name + ":<none>", image["id"]))
        return images

    def fmt_container_cmd(self, container, cmd, quotecmd):
        """Format a command to run inside a container using the runtime

        :param container: The name or ID of the container in which to run
        :type container: ``str``

        :param cmd: The command to run inside `container`
        :type cmd: ``str``

        :param quotecmd: Whether the cmd should be quoted.
        :type quotecmd: ``bool``

        :returns: Formatted string to run `cmd` inside `container`
        :rtype: ``str``
        """
        if quotecmd:
            quoted_cmd = quote(cmd)
        else:
            quoted_cmd = cmd
        container_id = self.get_container_by_name(container)
        return (f"{self.run_cmd} {container_id} {quoted_cmd}"
                if container_id is not None else '')

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import unittest

from sos.utilities import ImporterHelper


class ImporterHelperTests(unittest.TestCase):

    def test_runs(self):
        h = ImporterHelper(unittest)
        modules = h.get_modules()
        self.assertTrue('main' in modules)


if __name__ == "__main__":
    unittest.main()

# vim: set et ts=4 sw=4 :



# Copyright (C) 2016 Jorge Niedbalski <niedbalski@ubuntu.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json
from sos.report.plugins import Plugin, UbuntuPlugin, SoSPredicate


class LXD(Plugin, UbuntuPlugin):

    short_desc = 'LXD container hypervisor'
    plugin_name = 'lxd'
    profiles = ('container',)
    packages = ('lxd',)
    commands = ('lxc', 'lxd',)
    services = ('snap.lxd.daemon', 'snap.lxd.activate')

    def setup(self):
        if self.is_snap:

            lxd_pred = SoSPredicate(self, services=['snap.lxd.daemon'],
                                    required={'services': 'all'})

            self.add_cmd_output("lxd.buginfo", pred=lxd_pred, snap_cmd=True)

            self.add_copy_spec([
                '/var/snap/lxd/common/config',
                '/var/snap/lxd/common/global-conf',
                '/var/snap/lxd/common/lxc/local.conf',
                '/var/snap/lxd/common/lxd/logs/*/*.conf',
            ])

            if not self.get_option("all_logs"):
                self.add_copy_spec([
                    '/var/snap/lxd/common/lxd/logs/*.log',
                    '/var/snap/lxd/common/lxd/logs/*/*.log',
                ])
            else:
                self.add_copy_spec([
                    '/var/snap/lxd/common/lxd/logs/**',
                ])

            dqlite_crt = "/var/snap/lxd/common/lxd/cluster.crt"
            self.add_cmd_output(
                f"openssl x509 -in {dqlite_crt} -noout -dates",
            )

            db_path = "/var/snap/lxd/common/lxd/database/global"
            self.add_dir_listing(
                db_path,
                suggest_filename="ls_lxd_dqlite_dir",
            )

            queries = [
                {
                    "query": (
                        "SELECT * FROM sqlite_master WHERE type=\"table\";"
                    ),
                    "suggested_file_suffix": "schema",
                    "db": "local",
                },
                {
                    "query": (
                        "SELECT * FROM config WHERE NOT ( "
                        "key LIKE \"%keyring%\" OR "
                        "key LIKE \"%ca_cert%\" OR "
                        "key LIKE \"%ca_key%\" );"
                    ),
                    "suggested_file_suffix": "config",
                    "db": "local"
                },
                {
                    "query": "SELECT * FROM raft_nodes;",
                    "suggested_file_suffix": "raft_nodes",
                    "db": "local",
                },
                {
                    "query": "SELECT * FROM nodes;",
                    "suggested_file_suffix": "nodes",
                    "db": "global",
                },
                {
                    "query": "SELECT * FROM nodes_roles;",
                    "suggested_file_suffix": "nodes_roles",
                    "db": "global",
                },
            ]

            for query_entry in queries:
                db = query_entry.get("db", "local")
                query = json.dumps(query_entry.get("query"))
                file_suffix = query_entry.get("suggested_file_suffix")
                self.add_cmd_output(
                    f"lxd sql {db} {query}",
                    suggest_filename=f"lxd_sql_{db}_{file_suffix}",
                    pred=lxd_pred,
                )

        else:
            lxd_pred = SoSPredicate(self, services=['lxd'],
                                    required={'services': 'all'})
            self.add_copy_spec([
                "/etc/default/lxd-bridge",
                "/var/log/lxd/*"
            ])

            self.add_cmd_output([
                "lxc image list",
                "lxc list",
                "lxc network list",
                "lxc profile list",
                "lxc storage list"
            ], pred=lxd_pred)

            self.add_cmd_output([
                "find /var/lib/lxd -maxdepth 2 -type d -ls",
            ], suggest_filename='var-lxd-dirs.txt')

    def postproc(self):
        self.do_cmd_private_sub('lxd.buginfo')

# vim: set et ts=4 sw=4 :



# Copyright (C) 2015 Red Hat, Inc. Bryn M. Reeves <bmr@redhat.com>
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import os
import sys

from sos.report.plugins import RedHatPlugin, SuSEPlugin
from sos.policies.distros import LinuxPolicy
from sos.policies.package_managers.rpm import RpmPackageManager
from sos import _sos as _


class SuSEPolicy(LinuxPolicy):
    os_release_name = "SuSE"
    vendor = "SuSE"
    vendor_urls = [('Distribution Website', 'https://www.suse.com/')]
    _tmp_dir = "/var/tmp"

    def __init__(self, sysroot=None, init=None, probe_runtime=True,
                 remote_exec=None):
        super().__init__(sysroot=sysroot, init=init,
                         probe_runtime=probe_runtime,
                         remote_exec=remote_exec)
        self.valid_subclasses += [SuSEPlugin, RedHatPlugin]

        self.usrmove = False
        self.package_manager = RpmPackageManager()

        # If rpm query timed out after timeout duration exit
        if not self.package_manager.packages:
            self.ui_log.error("Could not obtain installed package list.")
            sys.exit(1)

        self.PATH = "/usr/sbin:/usr/bin:/root/bin:/sbin"
        self.PATH += os.pathsep + "/usr/local/bin"
        self.PATH += os.pathsep + "/usr/local/sbin"
        self.set_exec_path()

    @classmethod
    def check(cls, remote=''):
        """This method checks to see if we are running on SuSE. It must be
        overriden by concrete subclasses to return True when running on an
        OpenSuSE, SLES or other Suse distribution and False otherwise."""
        return False

    def get_tmp_dir(self, opt_tmp_dir):
        if not opt_tmp_dir:
            return self._tmp_dir
        return opt_tmp_dir

    def get_local_name(self):
        return self.host_name()


class OpenSuSEPolicy(SuSEPolicy):
    vendor = "SuSE"
    vendor_urls = [('Community Website', 'https://www.opensuse.org/')]
    os_release_name = "OpenSuSE"
    os_release_file = '/etc/SUSE-brand'
    msg = _("""\
This command will collect diagnostic and configuration \
information from this %(os_release_name)s system and installed \
applications.

An archive containing the collected information will be \
generated in %(tmpdir)s and may be provided to a %(vendor)s \
support representative.

No changes will be made to system configuration.
%(vendor_text)s
""")

    def __init__(self, sysroot=None, init=None, probe_runtime=True,
                 remote_exec=None):
        super().__init__(sysroot=sysroot, init=init,
                         probe_runtime=probe_runtime,
                         remote_exec=remote_exec)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2016 Red Hat, Inc.,Poornima M. Kshirsagar <pkshiras@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class OpenStackManila(Plugin):

    short_desc = 'OpenStack Manila'
    plugin_name = "openstack_manila"
    profiles = ('openstack', 'openstack_controller')
    containers = ('.*manila_api',)
    var_puppet_gen = "/var/lib/config-data/puppet-generated/manila"

    def setup(self):

        pre_conf = (
            self.var_puppet_gen if self.container_exists('.*manila_api') else
            '')
        config_dir = f"{pre_conf}/etc/manila"
        manila_cmd = f"manila-manage --config-dir {config_dir} db version"
        self.add_cmd_output(manila_cmd, suggest_filename="manila_db_version")

        self.add_copy_spec([
            "/etc/manila/",
            self.var_puppet_gen + "/etc/manila/",
            self.var_puppet_gen + "/etc/my.cnf.d/tripleo.cnf",
            self.var_puppet_gen + "/etc/httpd/conf/",
            self.var_puppet_gen + "/etc/httpd/conf.d/",
            self.var_puppet_gen + "/etc/httpd/conf.modules.d/*.conf",
        ])

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/manila/*",
            ])
        else:
            self.add_copy_spec([
                "/var/log/manila/*.log",
            ])

        self.add_file_tags({
            ".*/etc/manila/manila.conf": "manila_conf"
        })

    def apply_regex_sub(self, regexp, subst):
        """ Apply regex substitution """
        self.do_path_regex_sub("/etc/manila/*", regexp, subst)
        self.do_path_regex_sub(
            self.var_puppet_gen + "/etc/manila/*",
            regexp, subst
        )

    def postproc(self):
        protect_keys = [".*password.*", "transport_url",
                        "hdfs_ssh_pw", "maprfs_ssh_pw",
                        "memcache_secret_key"]
        connection_keys = ["connection", "sql_connection"]

        join_con_keys = "|".join(connection_keys)

        self.apply_regex_sub(
            fr"(^\s*({'|'.join(protect_keys)})\s*=\s*)(.*)",
            r"\1*********"
        )
        self.apply_regex_sub(
            fr"(^\s*({join_con_keys})\s*=\s*(.*)://(\w*):)(.*)(@(.*))",
            r"\1*********\6"
        )


class DebianManila(OpenStackManila, DebianPlugin, UbuntuPlugin):

    short_desc = 'OpenStack Manila information for Debian based distributions'
    packages = (
        'python-manila',
        'manila-common',
        'manila-api',
        'manila-share',
        'manila-scheduler',
        'python3-manila',
    )

    def setup(self):
        super().setup()
        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/apache2/manila*",
            ])
        else:
            self.add_copy_spec([
                "/var/log/apache2/manila*.log",
            ])


class RedHatManila(OpenStackManila, RedHatPlugin):

    short_desc = 'OpenStack Manila information for Red Hat distributions'
    packages = ('openstack-selinux',)

    def setup(self):
        super().setup()
        self.add_copy_spec("/etc/sudoers.d/manila")

        if self.get_option("all_logs"):
            self.add_copy_spec([
                "/var/log/containers/manila/*"
            ])
        else:
            self.add_copy_spec([
                "/var/log/containers/manila/*.log"
            ])


# vim: et ts=4 sw=4



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin, SoSPredicate
from sos.policies.distros.ubuntu import UbuntuPolicy


class Processor(Plugin, IndependentPlugin):

    short_desc = 'CPU information'

    plugin_name = 'processor'
    profiles = ('system', 'hardware', 'memory')
    files = ('/proc/cpuinfo',)
    packages = ('cpufreq-utils', 'cpuid')

    cpu_kmods = []

    def setup(self):

        cpupath = '/sys/devices/system/cpu'

        self.add_file_tags({
            f"{cpupath}/smt/control": 'cpu_smt_control',
            f"{cpupath}/smt/active": 'cpu_smt_active',
            f"{cpupath}/vulnerabilities/.*": 'cpu_vulns',
            f"{cpupath}/vulnerabilities/spectre_v2": 'cpu_vulns_spectre_v2',
            f"{cpupath}/vulnerabilities/meltdown": 'cpu_vulns_meltdown',
            f"{cpupath}/cpu.*/online": 'cpu_cores',
            f"{cpupath}/cpu/cpu0/cpufreq/cpuinfo_max_freq":
                'cpuinfo_max_freq'
        })

        self.add_copy_spec([
            "/proc/cpuinfo",
            "/sys/class/cpuid",
        ])
        # copy /sys/devices/system/cpu/cpuX with separately applied sizelimit
        # this is required for systems with tens/hundreds of CPUs where the
        # cumulative directory size exceeds 25MB or even 100MB.
        cdirs = self.listdir('/sys/devices/system/cpu')
        self.add_copy_spec([
            self.path_join('/sys/devices/system/cpu', cdir) for cdir in cdirs
        ])

        self.add_cmd_output([
            "lscpu",
            "lscpu -ae",
            "cpufreq-info",
            "cpuid",
            "cpuid -r",
        ], cmd_as_tag=True)

        if (isinstance(self.policy, UbuntuPolicy) and
                self.policy.dist_version() >= 20.04):
            self.cpu_kmods = ['msr']

        cpupower_pred = SoSPredicate(self, kmods=self.cpu_kmods)

        self.add_cmd_output([
            "cpupower frequency-info",
            "cpupower info",
            "cpupower idle-info",
        ], cmd_as_tag=True, pred=cpupower_pred)

        self.add_cmd_output("turbostat --debug sleep 10", cmd_as_tag=True,
                            pred=cpupower_pred, timeout=15)

        if '86' in self.policy.get_arch():
            self.add_cmd_output("x86info -a")


# vim: set et ts=4 sw=4 :



# Copyright (C) 2019 Red Hat, Inc., Jake Hunsaker <jhunsake@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class RHCoreOS(Plugin, RedHatPlugin):

    short_desc = 'Red Hat CoreOS'

    plugin_name = 'rhcos'
    packages = ('afterburn', 'redhat-release-coreos')

    def setup(self):
        units = ['coreos-boot-edit', 'coreos-copy-firstboot-network',
                 'coreos-generate-iscsi-initiatorname',
                 'coreos-gpt-setup', 'coreos-teardown-initramfs',
                 'gcp-routes', 'ignition-disks', 'ignition-fetch',
                 'ignition-fetch-offline', 'ignition-files',
                 'ignition-firstboot-complete', 'ignition-mount',
                 'ignition-ostree-growfs', 'ignition-ostree-populate-var',
                 'ignition-remount-system', 'ignition-setup-user']

        for unit in units:
            self.add_journal(unit)

        self.add_cmd_output(
            'afterburn --cmdline --attributes /dev/stdout',
            timeout=60
        )

# vim: set et ts=4 sw=4 :



# Copyright (C) 2007 Red Hat, Inc., Eugene Teo <eteo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import pwd
from glob import glob
from sos.report.plugins import Plugin, IndependentPlugin, PluginOpt


class Ssh(Plugin, IndependentPlugin):

    short_desc = 'Secure shell service'

    plugin_name = 'ssh'
    profiles = ('services', 'security', 'system', 'identity')

    option_list = [
        PluginOpt('userconfs', default=False,
                  desc=('Changes whether module will '
                        'collect user .ssh configs'))
    ]

    def setup(self):

        self.add_file_tags({
            '/etc/ssh/sshd_config$': 'sshd_config',
            '/etc/ssh/ssh_config$': 'ssh_config'
        })

        sshcfgs = [
            "/etc/ssh/ssh_config",
            "/etc/ssh/sshd_config",
            "/etc/ssh/sshd_config.d/*",
            ]

        # Include main config files
        self.add_copy_spec(sshcfgs)

        self.included_configs(sshcfgs)

        # If userconfs option is set to False, skips this
        if self.get_option('userconfs'):
            self.user_ssh_files_permissions()

        # Capture the output of extended test mode
        self.add_cmd_output("sshd -T")

    def included_configs(self, sshcfgs):
        """ Include subconfig files """
        # Read configs for any includes and copy those
        try:
            cfgfiles = [
                f for files in [
                    glob(copyspec, recursive=True) for copyspec in sshcfgs
                ] for f in files
            ]
            for sshcfg in cfgfiles:
                tag = sshcfg.split('/')[-1]
                with open(self.path_join(sshcfg), 'r',
                          encoding='UTF-8') as cfgfile:
                    for line in cfgfile:
                        # skip empty lines and comments
                        if len(line.split()) == 0 or line.startswith('#'):
                            continue
                        # ssh_config keywords are allowed as case-insensitive
                        if line.lower().startswith('include'):
                            confarg = line.split()
                            self.add_copy_spec(confarg[1], tags=tag)
        except Exception:  # pylint: disable=broad-except
            pass

    def user_ssh_files_permissions(self):
        """
        Iterate over .ssh folders in user homes to see their permissions.

        Bad permissions can prevent SSH from allowing access to given user.
        """
        users_data = pwd.getpwall()

        fs_mount_info = {}
        try:
            with open('/proc/mounts', "r", encoding='UTF-8') as mounts_file:
                for line in mounts_file:
                    (fs_file, fs_vstype) = line.split()[1:3]
                    fs_mount_info[fs_file] = fs_vstype
        except Exception:
            self._log_error("Couldn't read /proc/mounts")
            return
        non_local_fs = {'nfs', 'nfs4', 'autofs'}
        # Read the home paths of users in the system and check the ~/.ssh dirs
        for user in users_data:
            if user.pw_dir in fs_mount_info and \
                    fs_mount_info[user.pw_dir] in non_local_fs:
                self._log_info(
                        f"Skipping capture in {user.pw_dir}"
                        " because it's a remote directory"
                )
                continue
            home_dir = self.path_join(user.pw_dir, '.ssh')
            self.add_dir_listing(home_dir)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2018 Red Hat, Inc., Pavel Moravec <pmoravec@redhat.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program; if not, see <https://www.gnu.org/licenses/>.

from os.path import join
from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin


class OmnipathClient(Plugin, RedHatPlugin, UbuntuPlugin):

    short_desc = 'OmniPath Tools and Fast Fabric Client'

    plugin_name = 'omnipath_client'
    profiles = ('hardware',)

    packages = ('opa-basic-tools',)

    def setup(self):

        self.add_cmd_output([
            "opainfo",
            "opafabricinfo",
            "opahfirev",
            "opapmaquery",
            "opaportinfo",
            "opasaquery",
            "opasmaquery",
            "opashowmc",
            "opareports",
        ])

        # opacapture generates a tarball of given name we should collect;
        # rather than storing it somewhere under /var/tmp and copying it via
        # add_copy_spec, add it directly to sos_commands/<plugin> dir by
        # building a path argument using self.get_cmd_output_path().
        # This command calls 'depmod -a', so lets make sure we
        # specified the 'allow-system-changes' option before running it.
        if self.get_option('allow_system_changes'):
            opa_fullpath = join(self.get_cmd_output_path(), "opacapture.tgz")
            self.add_cmd_output(f"opacapture {opa_fullpath}",
                                changes=True)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2009 Red Hat, Inc., Joey Boggs <jboggs@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.


from sos.report.plugins import Plugin, IndependentPlugin


class Kvm(Plugin, IndependentPlugin):

    short_desc = 'Kernel virtual machine'

    plugin_name = 'kvm'
    profiles = ('system', 'virt')
    files = ('/dev/kvm',)

    def setup(self):
        self.add_copy_spec([
            "/sys/module/kvm/srcversion",
            "/sys/module/kvm_intel/srcversion",
            "/sys/module/kvm_amd/srcversion",
            "/sys/module/ksm/srcversion"
        ])

        self.add_cmd_output("kvm_stat --once")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, UbuntuPlugin


class NetworkManager(Plugin, RedHatPlugin, UbuntuPlugin):

    short_desc = 'NetworkManager service configuration'

    plugin_name = 'networkmanager'
    profiles = ('network', 'hardware', 'system')
    packages = ('NetworkManager', 'network-manager')

    def setup(self):
        self.system_connection_files = [
            "/etc/NetworkManager/system-connections/",
            "/usr/lib/NetworkManager/system-connections/",
            "/run/NetworkManager/system-connections/",
            "/var/run/NetworkManager/system-connections/",
            "/var/run/NetworkManager/backups/",
        ]

        self.add_copy_spec(self.system_connection_files)

        self.add_copy_spec([
            "/etc/NetworkManager/NetworkManager.conf",
            "/etc/NetworkManager/dispatcher.d",
            "/etc/NetworkManager/conf.d",
            "/usr/lib/NetworkManager/conf.d",
            "/run/NetworkManager/conf.d",
            "/var/lib/NetworkManager/NetworkManager-intern.conf",
            "/var/run/NetworkManager",
        ])

        self.add_forbidden_path(
            "/var/run/NetworkManager/secret_key"
        )

        self.add_journal(units="NetworkManager")

        self.add_cmd_output("NetworkManager --print-config")

        # There are some incompatible changes in nmcli since
        # the release of NetworkManager >= 0.9.9. In addition,
        # NetworkManager >= 0.9.9 will use the long names of
        # "nmcli" objects.

        # All versions conform to the following templates with different
        # strings for the object being operated on.
        nmcli_con_details_template = "nmcli con %s id"
        nmcli_dev_details_template = "nmcli dev %s"

        # test NetworkManager status for the specified major version
        def test_nm_status(version=1):
            status_template = "nmcli --terse --fields RUNNING %s status"
            obj_table = [
                "nm",        # <  0.9.9
                "general"    # >= 0.9.9
            ]
            status = self.exec_cmd(status_template % obj_table[version])
            return (status['status'] == 0 and
                    status['output'].lower().startswith("running"))

        # NetworkManager >= 0.9.9 (Use short name of objects for nmcli)
        if test_nm_status(version=1):
            self.add_cmd_output([
                "nmcli general status",
                "nmcli con",
                "nmcli -f all con",
                "nmcli con show --active",
                "nmcli dev"])
            nmcli_con_details_cmd = nmcli_con_details_template % "show"
            nmcli_dev_details_cmd = nmcli_dev_details_template % "show"

        # NetworkManager < 0.9.9 (Use short name of objects for nmcli)
        elif test_nm_status(version=0):
            self.add_cmd_output([
                "nmcli nm status",
                "nmcli con",
                "nmcli con status",
                "nmcli dev"])
            nmcli_con_details_cmd = nmcli_con_details_template % "list id"
            nmcli_dev_details_cmd = nmcli_dev_details_template % "list iface"

        # No grokkable NetworkManager version present
        else:
            nmcli_con_details_cmd = ""
            nmcli_dev_details_cmd = ""

        if len(nmcli_con_details_cmd) > 0:
            nmcli_con_show_result = self.exec_cmd(
                "nmcli --terse --fields NAME con"
            )
            if nmcli_con_show_result['status'] == 0:
                for con in nmcli_con_show_result['output'].splitlines():
                    if con[0:7] == 'Warning':
                        continue
                    # nm names may contain embedded quotes (" and '). These
                    # will cause an exception in shlex.split() if the quotes
                    # are unbalanced. This may happen with names like:
                    # "Foobar's Wireless Network". Although the problem will
                    # occur for both single and double quote characters the
                    # former is considerably more likely in object names since
                    # it is syntactically valid in many human languages.
                    #
                    # Reverse the normal sos quoting convention here and place
                    # double quotes around the innermost quoted string.
                    self.add_cmd_output(f'{nmcli_con_details_cmd} "{con}"')

            self.add_device_cmd(
                nmcli_dev_details_cmd + ' "%(dev)s"',
                devices='ethernet'
            )

        self.add_cmd_tags({
            "nmcli dev show": "nmcli_dev_show",
            "nmcli dev show .*": "nmcli_dev_show_sos"
        })

    def postproc(self):
        for sc_path in self.system_connection_files:
            self.do_path_regex_sub(
                f"{sc_path}",
                r"(password|psk|mka-cak|password-raw|pin|preshared-key"
                r"|private-key|secrets|wep-key[0-9])=(.*)",
                r"\1=***",
            )

# vim: set et ts=4 sw=4 :



# Copyright (C) 2024 Canonical Ltd., Arif Ali <arif.ali@canonical.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import json
import pwd
import yaml
from sos.report.plugins import Plugin, UbuntuPlugin, PluginOpt


class Sunbeam(Plugin, UbuntuPlugin):

    short_desc = "Sunbeam Openstack Controller"

    plugin_name = "sunbeam"
    profiles = ('cloud',)
    packages = ('openstack',)

    common_dir = '/var/snap/openstack/common'

    option_list = [
        PluginOpt('sunbeam-user', default='ubuntu', val_type=str,
                  desc='The user used for sunbeam installation'),
        PluginOpt('juju-allow-login', default=False, val_type=bool,
                  desc='Allow sos to login to juju'),
    ]

    def setup(self):

        self.add_service_status('snap.openstack.*')

        self.add_copy_spec([
            f'{self.common_dir}/hooks.log',
            f'{self.common_dir}/state/daemon.yaml',
            f'{self.common_dir}/state/truststore/sunbeam.maas.yaml',
            f'{self.common_dir}/state/database/info.yaml',
            f'{self.common_dir}/state/database/cluster.yaml',
            '/var/snap/openstack/current/config.yaml',
        ])

        sunbeam_user = self.get_option("sunbeam-user")
        try:
            user_pwd = pwd.getpwnam(sunbeam_user)
        except KeyError:
            # The user doesn't exist, this will skip the rest
            self._log_warn(
                f'User "{sunbeam_user}" does not exist, will not collect juju '
                'information. Use `-k sunbeam.sunbeam-user` option to define '
                'the user to use to collect data for sunbeam')
            return

        if user_pwd:
            self.add_cmd_output([
                'sunbeam cluster list',
                'sunbeam cluster list --format yaml',
                'sunbeam manifest list',
                'sunbeam deployment list',
            ], snap_cmd=True, runas=sunbeam_user)

            manifest_raw = self.collect_cmd_output(
                'sunbeam manifest list --format yaml',
                runas=sunbeam_user
            )

            if manifest_raw['status'] == 0:
                manifests = yaml.safe_load(manifest_raw['output'])
                for manifest in manifests:
                    self.add_cmd_output(
                        f'sunbeam manifest show {manifest["manifestid"]}',
                        snap_cmd=True, runas=sunbeam_user
                    )

            deployment_raw = self.collect_cmd_output(
                'sunbeam deployment list --format yaml',
                runas=sunbeam_user
            )

            if deployment_raw['status'] == 0:
                deployments = yaml.safe_load(deployment_raw['output'])
                for deployment in deployments['deployments']:
                    self.add_cmd_output([
                        f'sunbeam deployment show {deployment["name"]}',
                        f'sunbeam deployment show {deployment["name"]} '
                        '--format yaml',
                    ], snap_cmd=True, runas=sunbeam_user)

            sb_snap_homedir = f'{user_pwd.pw_dir}/snap/openstack/common'

            self.add_copy_spec([
                f"{sb_snap_homedir}/*.log",
                f"{sb_snap_homedir}/etc/**/*.log",
                f"{sb_snap_homedir}/etc/**/terraform.tfvars.json",
                f"{sb_snap_homedir}/logs/*.log",
                f"{sb_snap_homedir}/reports/*.yaml",
            ])

            if self.get_option("juju-allow-login"):
                self.exec_cmd(
                    f'su - {sunbeam_user} -c "sunbeam utils juju-login"')

            # This checks if the juju user is logged in, and if it is, then we
            # collect the juju information. It could be that the user was
            # already logged in from a prior session
            juju_whoami = self.exec_cmd('juju whoami', runas=sunbeam_user)
            juju_status = self.exec_cmd('juju status', runas=sunbeam_user,
                                        timeout=30)
            logged_in = False
            if juju_whoami['status'] == 0 or juju_status['status'] == 0:
                try:
                    j_whoami = juju_whoami['output'].splitlines()[0]
                    j_status = juju_status['output'].splitlines()[0]

                    if "Controller" in j_whoami or "Controller" in j_status:
                        self._get_juju_cmd_details(sunbeam_user)
                        logged_in = True
                except IndexError:
                    # One of the commands may not have gone through and hence
                    # not logged in
                    pass

            if not logged_in:
                self._log_warn(
                    "juju is not logged in, will not collect juju "
                    "information. Use `-k sunbeam.juju-allow-login=True` to "
                    "login or use `juju login` as the sunbeam user to "
                    "login")

    def _get_juju_cmd_details(self, user):
        self.add_cmd_output("juju controllers", runas=user, snap_cmd=True)
        juju_controllers = self.collect_cmd_output(
            "juju controllers --format json", runas=user)

        if juju_controllers['status'] == 0:
            juju_ctrl_json = json.loads(juju_controllers['output'])
            for controller in juju_ctrl_json['controllers'].keys():

                self.add_cmd_output([
                    f'juju models -c {controller}',
                    f'juju model-defaults -c {controller}',
                    f'juju controller-config -c {controller}',
                    f'juju controller-config -c {controller} --format json',
                ], runas=user, snap_cmd=True)

                juju_models = self.collect_cmd_output(
                    f'juju models -c {controller} --format json',
                    runas=user)

                if juju_models['status'] == 0:
                    juju_status_json = json.loads(juju_models['output'])

                    for model in juju_status_json['models']:

                        model_name = f'{controller}:{model["name"]}'

                        self.add_cmd_output([
                            f'juju status -m {model_name}',
                            f'juju status -m {model_name} --format json',
                            f'juju model-config -m {model_name}',
                            f'juju model-config -m {model_name} --format json',
                        ], runas=user, snap_cmd=True)

    def postproc(self):

        self.do_file_private_sub(
            f'{self.common_dir}/state/truststore/sunbeam.maas.yaml'
        )

        self.do_cmd_private_sub('juju controllers')
        self.do_cmd_private_sub('juju controller-config')

# vim: et ts=4 sw=4



# Copyright (C) 2021 Red Hat, Inc., Jose Castillo <jcastillo@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Discovery(Plugin, RedHatPlugin):

    short_desc = 'Discovery inspection and reporting tool'
    plugin_name = 'discovery'
    packages = ('discovery', 'discovery-tools',)
    containers = ('dsc-db', 'discovery', 'discovery-toolbox')

    def setup(self):
        self.add_copy_spec([
            "/root/discovery/db/volume/data/userdata/pg_log/",
            "/root/discovery/server/volumes/log/app.log",
            "/root/discovery/server/volumes/log/discovery-server.log",
            "/var/lib/containers/storage/volumes/dsc-data/_data/userdata/log/",
            "/var/discovery/server/volumes/log/",
        ])

        self.add_container_logs(list(self.containers))
# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re


from avocado.utils import process
from sos_tests import StageOneReportTest, redhat_only, debian_only, ubuntu_only


# These are the header strings in --list-plugins output
DISABLED = 'The following plugins are currently disabled:'
OPTIONS = 'The following options are available for ALL plugins:'


class AllPluginSmokeTest(StageOneReportTest):
    """This test forcibly enables ALL plugins available to the local branch
    and aims to make sure that there are no exceptions generated during the
    execution

    :avocado: tags=stageone
    """

    def pre_sos_setup(self):
        _cmd = f'{self.sos_bin} report --list-plugins'
        out = process.run(_cmd, timeout=300).stdout.decode()
        reg = DISABLED + '(.*?)' + OPTIONS
        self.plugs = []
        for result in re.findall(reg, out, re.S):
            for line in result.splitlines():
                try:
                    self.plugs.append(line.split()[0])
                except Exception:
                    pass

        self.sos_cmd = f'-e {",".join(p for p in self.plugs)}'

    def test_all_plugins_ran(self):
        for plugin in self.plugs:
            self.assertPluginIncluded(plugin)

    @redhat_only
    def test_expected_warnings_displayed(self):
        """We can expect specific plugins to always generate a warning during
        setup if they are enabled on systems that are not configured for those
        plugins.

        Make sure our warnings are displayed
        """
        self.assertOutputContains('Source the environment file for the user '
                                  'intended to connect to the OpenStack '
                                  'environment.')
        self.assertOutputContains('Some or all of the skydive params are not '
                                  'set properly.')


class ExpectedDefaultPluginsTest(StageOneReportTest):
    """Make sure that a default expected set of plugins runs on a "normal"
    execution that does not provide any plugin-related options

    :avocado: tags=stageone
    """

    sos_cmd = ' '

    def test_default_plugins_enabled(self):
        """These plugins should run on all supported hosts by default everytime
        """
        self.assertPluginIncluded([
            'boot',
            'date',
            'filesys',
            'host',
            'kernel',
            'login',
            'logs',
            'pci',
            'process',
            'processor',
            'python',
            'release',
            'services',
            'udev',
            'usb'
        ])

    @redhat_only
    def test_rhel_default_plugins(self):
        """Plugins expected to always run on a RHEL (-like) system
        """
        self.assertPluginIncluded([
            'dnf',
            'rpm',
            'selinux',
            'unpackaged',
        ])

    @debian_only
    def test_debian_default_plugins(self):
        """Plugins expected to always run on a Debian (-like) system
        """
        self.assertPluginIncluded([
            'apparmor',
            'apt',
        ])

    @ubuntu_only
    def test_ubuntu_default_plugins(self):
        """Plugins expected to always run on a Ubuntu (-like) system
        """
        self.assertPluginIncluded([
            'ubuntu',
        ])



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Squid(Plugin):

    short_desc = 'Squid caching proxy'

    plugin_name = 'squid'
    profiles = ('webserver', 'services', 'sysmgmt')


class RedHatSquid(Squid, RedHatPlugin):

    files = ('/etc/squid/squid.conf',)
    packages = ('squid',)

    def setup(self):
        self.add_copy_spec([
            "/etc/squid/squid.conf",
            "/var/log/squid/access.log*",
            "/var/log/squid/cache.log*",
            "/var/log/squid/squid.out*"
        ])


class DebianSquid(Squid, DebianPlugin, UbuntuPlugin):

    plugin_name = 'squid'
    files = ('/etc/squid/squid.conf',)
    packages = ('squid',)

    def setup(self):
        self.add_copy_spec([
            "/etc/squid/squid.conf",
            "/var/log/squid/*",
            "/etc/squid-deb-proxy",
            "/var/log/squid-deb-proxy/*",
        ])

# vim: set et ts=4 sw=4 :



# Copyright (C) 2017 Red Hat, Inc., Bryn M. Reeves <bmr@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Crypto(Plugin, IndependentPlugin):

    short_desc = 'System crypto services information'

    plugin_name = 'crypto'
    profiles = ('system', 'hardware')

    def setup(self):

        cpth = '/etc/crypto-policies/back-ends'

        self.add_file_tags({
            f"{cpth}/bind.config": 'crypto_policies_bind',
            f"{cpth}/opensshserver.config": 'crypto_policies_opensshserver',
            '/etc/crypto-policies/.*/current': 'crypto_policies_state_current',
            '/etc/crypto-policies/config': 'crypto_policies_config'
        })

        self.add_copy_spec([
            "/proc/crypto",
            "/proc/sys/crypto/fips_enabled",
            "/etc/system-fips",
            "/etc/crypto-policies/*"
        ])

        self.add_cmd_output([
            "fips-mode-setup --check",
            "update-crypto-policies --show",
            "update-crypto-policies --is-applied"
        ])

# vim: et ts=4 sw=4



# Copyright (C) 2024 Red Hat, Inc., Jose Castillo <jcastillo@redhat.com>
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin


class Bootc(Plugin, RedHatPlugin):

    short_desc = 'Bootc'

    plugin_name = 'bootc'
    profiles = ('system', 'sysmgmt', 'packagemanager',)

    packages = ('bootc',)

    def setup(self):
        self.add_copy_spec([
            "/usr/lib/ostree/prepare-root.conf",
            "/usr/lib/bootc/",
        ])

        self.add_cmd_output(
            "bootc status",
        )

        self.add_forbidden_path("/usr/lib/bootc/storage")

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos_tests import StageTwoReportTest


class PluginTimeoutTest(StageTwoReportTest):
    """Test that whole plugin timeout control is working

    :avocado: tags=stagetwo
    """

    install_plugins = ['timeout_test']
    sos_cmd = '-o timeout_test -vvv --plugin-timeout=10'

    def test_correct_plugin_timeout(self):
        man = self.get_plugin_manifest('timeout_test')
        self.assertEqual(man['timeout'], 10)

    def test_plugin_timed_out(self):
        self.assertSosLogNotContains('collected plugin \'timeout_test\' in')
        self.assertSosUILogContains('Plugin timeout_test timed out')

    def test_no_output_collected(self):
        self.assertFileNotExists('sos_commands/timeout_test/echo_out')


class NativeCmdTimeoutTest(StageTwoReportTest):
    """Test that the native timeout control for the plugin API is working

    :avocado: tags=stagetwo
    """

    install_plugins = ['timeout_test']
    sos_cmd = '-o timeout_test,host -vvv'

    def test_correct_plugin_timeout(self):
        man = self.get_plugin_manifest('timeout_test')
        self.assertEqual(man['timeout'], 100)
        hman = self.get_plugin_manifest('host')
        self.assertEqual(hman['timeout'], 300)

    def test_plugin_completed(self):
        self.assertSosLogContains('collected plugin \'timeout_test\' in')
        self.assertFileCollected('sos_commands/timeout_test/echo_good')

    def test_command_timed_out(self):
        self.assertSosLogContains(
            r"\[plugin:timeout_test\] command 'sleep 30' timed out after 10s"
        )
        self.assertFileCollected('sos_commands/timeout_test/sleep_30')


class MultipleTimeoutValues(NativeCmdTimeoutTest):
    """Test that our plugin timeout option priority is functioning correctly

    :avocado: tags=stagetwo
    """

    install_plugins = ['timeout_test']
    sos_cmd = ('-o timeout_test,host --plugin-timeout=30 -k '
               'timeout_test.timeout=60')

    def test_correct_plugin_timeout(self):
        man = self.get_plugin_manifest('timeout_test')
        self.assertEqual(man['timeout'], 60)
        hman = self.get_plugin_manifest('host')
        self.assertEqual(hman['timeout'], 30)



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.
import glob
import yaml

from sos.report.plugins import Plugin, IndependentPlugin


class SaltMaster(Plugin, IndependentPlugin):

    short_desc = 'Salt Master'

    plugin_name = 'saltmaster'
    profiles = ('sysmgmt',)

    packages = ('salt-master', 'salt-api',)

    def setup(self):
        self.collected_pillar_roots = []

        if self.get_option("all_logs"):
            self.add_copy_spec("/var/log/salt")
        else:
            self.add_copy_spec("/var/log/salt/master")

        self.add_copy_spec("/etc/salt")
        self.add_forbidden_path("/etc/salt/pki/*/*.pem")

        self.add_pillar_roots()
        self.add_cmd_output([
            "salt-master --version",
            "systemctl --full status salt-master",
            "systemctl --full status salt-api",
            "salt-key --list all",
            "salt-run jobs.list_jobs --out=yaml",
            "salt-run manage.list_state --out=yaml",
            "salt-run manage.list_not_state --out=yaml",
            "salt-run manage.joined --out=yaml",
        ], timeout=30)

    def add_pillar_roots(self):
        """ Collect pilliar_roots of all salt configs """
        cfgs = glob.glob("/etc/salt/master.d/*conf")
        main_cfg = "/etc/salt/master"

        if self.path_exists(main_cfg):
            cfgs.append(main_cfg)

        all_pillar_roots = []
        for cfg in cfgs:
            with open(cfg, "r", encoding='UTF-8') as file:
                try:
                    cfg_pillar_roots = (
                        yaml.safe_load(file).get("pillar_roots", {}).
                        get("base", [])
                    )
                except AttributeError:
                    cfg_pillar_roots = []
                all_pillar_roots.extend(cfg_pillar_roots)

        self.collected_pillar_roots = all_pillar_roots
        self.add_copy_spec(all_pillar_roots)

    def postproc(self):
        regexp = (
            r'(^\s*.*(pass|secret|(?<![A-z])key(?![A-z])|'
            r'api_?key|encryption_?key).*:\ ).+'
        )
        subst = r'\1******'

        self.do_path_regex_sub("/etc/salt/*", regexp, subst)

        for pillar_root in self.collected_pillar_roots:
            normalized_root = pillar_root if pillar_root.endswith('/') else (
                f"{pillar_root}/"
            )
            self.do_path_regex_sub(f"{normalized_root}*", regexp, subst)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2007-2012 Red Hat, Inc., Ben Turner <bturner@redhat.com>

# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin


class Iscsi(Plugin):

    short_desc = 'iSCSI initiator'

    plugin_name = "iscsi"
    profiles = ('storage',)

    def setup(self):
        var_puppet_gen = "/var/lib/config-data/puppet-generated/iscsid"
        self.add_copy_spec([
            "/etc/iscsi/iscsid.conf",
            "/etc/iscsi/initiatorname.iscsi",
            var_puppet_gen + "/etc/iscsi/initiatorname.iscsi",
            "/var/lib/iscsi"
        ])
        self.add_cmd_output([
            "iscsiadm -m session -P 3",
            "iscsiadm -m node -P 1",
            "iscsiadm -m iface -P 1",
            "iscsiadm -m node --op=show"
        ])

    def postproc(self):
        # Example for scrubbing node.session.auth.password
        #
        # node.session.auth.password = jfaiu1nNQJcsa,sti4lho'jZia=ia
        #
        # to
        #
        # node.session.auth.password = ********
        nodesessionpwd = r"(node\.session\.auth\.password(_in)?\s+=\s+)(\S+)"
        discoverypwd = (
                r"(discovery\.sendtargets\.auth\.password(_in)?"
                r"\s+=\s+)(\S+)"
                        )
        repl = r"\1********\n"
        self.do_path_regex_sub('/etc/iscsi/iscsid.conf', nodesessionpwd, repl)
        self.do_path_regex_sub('/etc/iscsi/iscsid.conf', discoverypwd, repl)
        self.do_path_regex_sub(
                '/var/lib/iscsi/', nodesessionpwd, repl)
        self.do_path_regex_sub(
                '/var/lib/iscsi/', discoverypwd, repl)


class RedHatIscsi(Iscsi, RedHatPlugin):
    """ RedHatPlugin's setup() will be invoked """

    packages = ('iscsi-initiator-utils',)


class DebianIscsi(Iscsi, DebianPlugin, UbuntuPlugin):

    packages = ('open-iscsi',)

# vim: set et ts=4 sw=4 :



# Copyright (C) 2025 Henry AlOudaimy <henry.oudaimy@gmail.com>
#
# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

import re
from sos.report.plugins import Plugin, IndependentPlugin


class OpenSearch(Plugin, IndependentPlugin):

    short_desc = 'OpenSearch service'
    plugin_name = 'opensearch'
    profiles = ('services', )

    packages = ('opensearch',)
    services = ('opensearch',)

    def get_hostname_port(self, opensearch_config_file):
        """ Get hostname and port number """
        hostname = "localhost"
        port = "9200"
        try:
            with open(opensearch_config_file, encoding='UTF-8') as fread:
                for line in fread:
                    network_host = re.search(r'(^network.host):(.*)', line)
                    network_port = re.search(r'(^http.port):(.*)', line)
                    if network_host and len(network_host.groups()) == 2:
                        hostname = network_host.groups()[-1].strip()
                        hostname = re.sub(r'"|\'', '', hostname)
                        continue
                    if network_port and len(network_port.groups()) == 2:
                        port = network_port.groups()[-1].strip()
        except Exception as err:  # pylint: disable=broad-except
            self._log_info(f"Failed to parse {opensearch_config_file}: {err}")
        return hostname, port

    def setup(self):
        opensearch_config_file = self.path_join(
            "/etc/opensearch/opensearch.yml"
        )
        self.add_copy_spec(opensearch_config_file)

        if self.get_option("all_logs"):
            self.add_copy_spec("/var/log/opensearch/*")
        else:
            self.add_copy_spec("/var/log/opensearch/*.log")

        host, port = self.get_hostname_port(opensearch_config_file)
        endpoint = host + ":" + port
        self.add_cmd_output([
                f"curl -X GET '{endpoint}/_cluster/settings?pretty'",
                f"curl -X GET '{endpoint}/_cluster/health?pretty'",
                f"curl -X GET '{endpoint}/_cluster/stats?pretty'",
                f"curl -X GET '{endpoint}/_cat/nodes?v'",
                f"curl -X GET '{endpoint}/_cat/indices'",
                f"curl -X GET '{endpoint}/_cat/shards'",
                f"curl -X GET '{endpoint}/_cat/aliases'",
        ])

# vim: set et ts=4 sw=4 :



# This file is part of the sos project: https://github.com/sosreport/sos
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# version 2 of the GNU General Public License.
#
# See the LICENSE file in the source distribution for further information.

from sos.report.plugins import Plugin, IndependentPlugin


class Devices(Plugin, IndependentPlugin):

    short_desc = 'devices specific commands'

    plugin_name = 'devices'
    profiles = ('system', 'hardware', 'boot', 'storage')
    packages = ('udev', 'systemd-udev')
    files = ('/dev',)

    def setup(self):
        self.add_cmd_output("udevadm info --export-db")

# vim: et ts=4 sw=4
